"""
    Tests for the saltutil state
"""

import inspect

import pytest

import salt.modules.saltutil as saltutil_module
import salt.states.saltutil as saltutil_state
from tests.support.mock import MagicMock, patch


@pytest.fixture
def configure_loader_modules():
    return {saltutil_state: {"__opts__": {"test": False}}}


def test_saltutil_sync_all_nochange():
    sync_output = {
        "clouds": [],
        "engines": [],
        "executors": [],
        "grains": [],
        "beacons": [],
        "utils": [],
        "returners": [],
        "modules": [],
        "renderers": [],
        "log_handlers": [],
        "thorium": [],
        "states": [],
        "sdb": [],
        "proxymodules": [],
        "output": [],
        "pillar": [],
        "matchers": [],
        "serializers": [],
        "wrapper": [],
        "tops": [],
    }
    state_id = "somename"
    state_result = {
        "changes": {},
        "comment": "No updates to sync",
        "name": "somename",
        "result": True,
    }

    mock_moduleout = MagicMock(return_value=sync_output)
    with patch.dict(saltutil_state.__salt__, {"saltutil.sync_all": mock_moduleout}):
        result = saltutil_state.sync_all(state_id, refresh=True)
        assert result == state_result


def test_saltutil_sync_all_test():
    sync_output = {
        "clouds": [],
        "engines": [],
        "executors": [],
        "grains": [],
        "beacons": [],
        "utils": [],
        "returners": [],
        "modules": [],
        "renderers": [],
        "log_handlers": [],
        "thorium": [],
        "states": [],
        "sdb": [],
        "proxymodules": [],
        "output": [],
        "pillar": [],
        "matchers": [],
        "serializers": [],
        "wrapper": [],
        "tops": [],
    }
    state_id = "somename"
    state_result = {
        "changes": {},
        "comment": "saltutil.sync_all would have been run",
        "name": "somename",
        "result": None,
    }

    mock_moduleout = MagicMock(return_value=sync_output)
    with patch.dict(saltutil_state.__salt__, {"saltutil.sync_all": mock_moduleout}):
        with patch.dict(saltutil_state.__opts__, {"test": True}):
            result = saltutil_state.sync_all(state_id, refresh=True)
            assert result == state_result


def test_saltutil_sync_all_change():
    sync_output = {
        "clouds": [],
        "engines": [],
        "executors": [],
        "grains": [],
        "beacons": [],
        "utils": [],
        "returners": [],
        "modules": ["modules.file"],
        "renderers": [],
        "log_handlers": [],
        "thorium": [],
        "states": ["states.saltutil", "states.ssh_auth"],
        "sdb": [],
        "proxymodules": [],
        "output": [],
        "pillar": [],
        "matchers": [],
        "serializers": [],
        "wrapper": [],
        "tops": [],
    }
    state_id = "somename"
    state_result = {
        "changes": {
            "modules": ["modules.file"],
            "states": ["states.saltutil", "states.ssh_auth"],
        },
        "comment": "Sync performed",
        "name": "somename",
        "result": True,
    }

    mock_moduleout = MagicMock(return_value=sync_output)
    with patch.dict(saltutil_state.__salt__, {"saltutil.sync_all": mock_moduleout}):
        result = saltutil_state.sync_all(state_id, refresh=True)
        assert result == state_result


def test_saltutil_sync_states_should_match_saltutil_module():
    module_functions = [
        f[0]
        for f in inspect.getmembers(saltutil_module, inspect.isfunction)
        if f[0].startswith("sync_")
    ]
    state_functions = [
        f[0]
        for f in inspect.getmembers(saltutil_state, inspect.isfunction)
        if f[0].startswith("sync_")
    ]
    for fn in module_functions:
        assert (
            fn in state_functions
        ), f"modules.saltutil.{fn} has no matching state in states.saltutil"



import salt.utils.timed_subprocess as timed_subprocess
from tests.support.unit import TestCase


class TestTimedSubprocess(TestCase):
    def test_timedproc_with_shell_true_and_list_args(self):
        """
        This test confirms the fix for the regression introduced in 1f7d50d.
        The TimedProc dunder init would result in a traceback if the args were
        passed as a list and shell=True was set.
        """
        p = timed_subprocess.TimedProc(["echo", "foo"], shell=True)
        del p  # Don't need this anymore



"""
Test the win_network util
"""

import pytest

import salt.utils.win_network as win_network

pytestmark = [
    pytest.mark.windows_whitelisted,
    pytest.mark.skip_unless_on_windows,
]


def test__get_ip_base_properties():
    interfaces = win_network._get_network_interfaces()
    for interface in interfaces:
        base_properties = win_network._get_ip_base_properties(interface)
        assert "ipv4_dhcp_enabled" in base_properties
        assert "ipv4_forwarding_enabled" in base_properties
        assert "ipv4_wins_enabled" in base_properties


def test_get_interface_info_dot_net_formatted():
    interfaces = win_network.get_interface_info_dot_net_formatted()
    for interface in interfaces:
        assert "description" in interfaces[interface]
        assert "hwaddr" in interfaces[interface]
        assert "up" in interfaces[interface]
        assert "dhcp_enabled" in interfaces[interface]["inet"][0]



import logging
import os
import shutil

import pytest

import salt.serializers.json as jsonserializer
import salt.serializers.msgpack as msgpackserializer
import salt.serializers.yaml as yamlserializer
import salt.states.file as filestate
import salt.utils.files
import salt.utils.json
import salt.utils.platform
import salt.utils.win_functions
import salt.utils.yaml
from tests.support.mock import MagicMock, patch

log = logging.getLogger(__name__)


@pytest.fixture
def configure_loader_modules():
    return {
        filestate: {
            "__env__": "base",
            "__salt__": {"file.manage_file": False},
            "__serializers__": {
                "yaml.serialize": yamlserializer.serialize,
                "yaml.seserialize": yamlserializer.serialize,
                "json.serialize": jsonserializer.serialize,
                "msgpack.serialize": msgpackserializer.serialize,
            },
            "__opts__": {"test": False, "cachedir": ""},
            "__instance_id__": "",
            "__low__": {},
            "__utils__": {},
        }
    }


# 'copy' function tests: 1
def test_copy(tmp_path):
    """
    Test if the source file exists on the system, copy it to the named file.
    """
    name = str(tmp_path / "salt")
    source = str(tmp_path / "salt" / "salt")
    user = "salt"
    group = "saltstack"

    ret = {"name": name, "result": False, "comment": "", "changes": {}}

    comt = "Must provide name to file.copy"
    ret.update({"comment": comt, "name": ""})
    assert filestate.copy_("", source) == ret

    mock_t = MagicMock(return_value=True)
    mock_f = MagicMock(return_value=False)
    mock_uid = MagicMock(side_effect=["", "1000", "1000"])
    mock_gid = MagicMock(side_effect=["", "1000", "1000"])
    mock_user = MagicMock(return_value=user)
    mock_grp = MagicMock(return_value=group)
    mock_io = MagicMock(side_effect=IOError)
    with patch.object(os.path, "isabs", mock_f):
        comt = f"Specified file {name} is not an absolute path"
        ret.update({"comment": comt, "name": name})
        assert filestate.copy_(name, source) == ret

    with patch.object(os.path, "isabs", mock_t):
        with patch.object(os.path, "exists", mock_f):
            comt = f'Source file "{source}" is not present'
            ret.update({"comment": comt, "result": False})
            assert filestate.copy_(name, source) == ret

        with patch.object(os.path, "exists", mock_t):
            with patch.dict(
                filestate.__salt__,
                {
                    "file.user_to_uid": mock_uid,
                    "file.group_to_gid": mock_gid,
                    "file.get_user": mock_user,
                    "file.get_group": mock_grp,
                    "file.get_mode": mock_grp,
                    "file.check_perms": mock_t,
                },
            ):

                # Group argument is ignored on Windows systems. Group is set
                # to user
                if salt.utils.platform.is_windows():
                    comt = "User salt is not available Group salt is not available"
                else:
                    comt = "User salt is not available Group saltstack is not available"
                ret.update({"comment": comt, "result": False})
                assert filestate.copy_(name, source, user=user, group=group) == ret

                comt1 = (
                    'Failed to delete "{}" in preparation for'
                    " forced move".format(name)
                )
                comt2 = (
                    'The target file "{}" exists and will not be '
                    "overwritten".format(name)
                )
                comt3 = f'File "{source}" is set to be copied to "{name}"'
                with patch.object(os.path, "isdir", mock_f):
                    with patch.object(os.path, "lexists", mock_t):
                        with patch.dict(filestate.__opts__, {"test": False}):
                            with patch.dict(
                                filestate.__salt__, {"file.remove": mock_io}
                            ):
                                ret.update({"comment": comt1, "result": False})
                                assert (
                                    filestate.copy_(
                                        name, source, preserve=True, force=True
                                    )
                                    == ret
                                )

                            with patch.object(os.path, "isfile", mock_t):
                                ret.update({"comment": comt2, "result": True})
                                assert (
                                    filestate.copy_(name, source, preserve=True) == ret
                                )

                    with patch.object(os.path, "lexists", mock_f):
                        with patch.dict(filestate.__opts__, {"test": True}):
                            ret.update({"comment": comt3, "result": None})
                            assert filestate.copy_(name, source, preserve=True) == ret

                        with patch.dict(filestate.__opts__, {"test": False}):
                            comt = "The target directory {} is not present".format(
                                tmp_path
                            )
                            ret.update({"comment": comt, "result": False})
                            assert filestate.copy_(name, source, preserve=True) == ret

            check_perms_ret = {
                "name": name,
                "changes": {},
                "comment": [],
                "result": True,
            }
            check_perms_perms = {}

            if salt.utils.platform.is_windows():
                mock_check_perms = MagicMock(return_value=check_perms_ret)
            else:
                mock_check_perms = MagicMock(
                    return_value=(check_perms_ret, check_perms_perms)
                )
            with patch.dict(
                filestate.__salt__,
                {
                    "file.user_to_uid": mock_uid,
                    "file.group_to_gid": mock_gid,
                    "file.get_user": mock_user,
                    "file.get_group": mock_grp,
                    "file.get_mode": mock_grp,
                    "file.check_perms": mock_check_perms,
                },
            ):

                comt = f'Copied "{source}" to "{name}"'
                with patch.dict(filestate.__opts__, {"user": "salt"}), patch.object(
                    os.path, "isdir", mock_t
                ), patch.object(os.path, "lexists", mock_f), patch.dict(
                    filestate.__opts__, {"test": False}
                ), patch.dict(
                    filestate.__salt__, {"file.remove": mock_io}
                ), patch.object(
                    shutil, "copytree", MagicMock()
                ):
                    group = None
                    ret.update(
                        {
                            "comment": comt,
                            "result": True,
                            "changes": {name: source},
                        }
                    )
                    res = filestate.copy_(name, source, group=group, preserve=False)
                    assert res == ret

                comt = f'Copied "{source}" to "{name}"'
                with patch.dict(filestate.__opts__, {"user": "salt"}), patch.object(
                    os.path, "isdir", MagicMock(side_effect=[False, True, False])
                ), patch.object(os.path, "lexists", mock_f), patch.dict(
                    filestate.__opts__, {"test": False}
                ), patch.dict(
                    filestate.__salt__, {"file.remove": mock_io}
                ), patch.object(
                    shutil, "copy", MagicMock()
                ):
                    group = None
                    ret.update(
                        {
                            "comment": comt,
                            "result": True,
                            "changes": {name: source},
                        }
                    )
                    res = filestate.copy_(name, source, group=group, preserve=False)
                    assert res == ret


def test_copy_test_mode_user_group_not_present():
    """
    Test file copy in test mode with no user or group existing
    """
    source = "/tmp/src_copy_no_user_group_test_mode"
    filename = "/tmp/copy_no_user_group_test_mode"
    with patch.dict(
        filestate.__salt__,
        {
            "file.group_to_gid": MagicMock(side_effect=["1234", "", ""]),
            "file.user_to_uid": MagicMock(side_effect=["", "4321", ""]),
            "file.get_mode": MagicMock(return_value="0644"),
        },
    ), patch.dict(filestate.__opts__, {"test": True}), patch.object(
        os.path, "exists", return_value=True
    ):
        ret = filestate.copy_(
            source, filename, group="nonexistinggroup", user="nonexistinguser"
        )
        assert ret["result"] is not False
        assert "is not available" not in ret["comment"]

        ret = filestate.copy_(
            source, filename, group="nonexistinggroup", user="nonexistinguser"
        )
        assert ret["result"] is not False
        assert "is not available" not in ret["comment"]

        ret = filestate.copy_(
            source, filename, group="nonexistinggroup", user="nonexistinguser"
        )
        assert ret["result"] is not False
        assert "is not available" not in ret["comment"]



"""
unit tests for the reactor runner
"""

import logging

import pytest

import salt.runners.reactor as reactor
from salt.exceptions import CommandExecutionError
from salt.utils.event import SaltEvent
from tests.support.mock import MagicMock, patch

log = logging.getLogger(__name__)

pytestmark = [
    pytest.mark.usefixtures("mocked_tcp_pub_client"),
]


class MockEvent:
    """
    Mock event class
    """

    flag = None

    def __init__(self):
        self.full = None

    def get_event(self, wait, tag):
        """
        Mock get_event method
        """
        data = []
        return {"tag": tag, "data": data}

    def fire_event(self, data, tag):
        """
        Mock get_event method
        """
        return {"tag": tag, "data": data}


@pytest.fixture
def configure_loader_modules():
    return {
        reactor: {
            "__opts__": {
                "reactor": [],
                "engines": [],
                "id": "master_id",
                "sock_dir": "/var/run/salt/master",
                "transport": "zeromq",
            },
            "__jid_event__": MockEvent(),
        }
    }


def test_list():
    """
    test reactor.list runner
    """
    with pytest.raises(CommandExecutionError) as excinfo:
        ret = reactor.list_()
    assert excinfo.value.error == "Reactor system is not running."

    mock_opts = {}
    mock_opts = {"engines": []}
    with patch.dict(reactor.__opts__, mock_opts):
        with pytest.raises(CommandExecutionError) as excinfo:
            ret = reactor.list_()
        assert excinfo.value.error == "Reactor system is not running."

    mock_opts = {}
    mock_opts["engines"] = [
        {
            "reactor": {
                "refresh_interval": 60,
                "worker_threads": 10,
                "worker_hwm": 10000,
            }
        }
    ]
    event_returns = {
        "reactors": [{"test_event/*": ["/srv/reactors/reactor.sls"]}],
        "_stamp": "2020-09-04T16:51:52.577711",
    }

    with patch.dict(reactor.__opts__, mock_opts):
        with patch.object(SaltEvent, "connect_pub", return_value=True):
            with patch.object(SaltEvent, "get_event", return_value=event_returns):
                with patch("salt.utils.master.get_master_key") as get_master_key:
                    get_master_key.retun_value = MagicMock(retun_value="master_key")
                    ret = reactor.list_()
                    assert {"test_event/*": ["/srv/reactors/reactor.sls"]} in ret

    event_returns = {
        "_stamp": "2020-09-04T16:51:52.577711",
    }

    with patch.dict(reactor.__opts__, mock_opts):
        with patch.object(SaltEvent, "connect_pub", return_value=True):
            with patch.object(SaltEvent, "get_event", return_value=event_returns):
                with patch("salt.utils.master.get_master_key") as get_master_key:
                    get_master_key.retun_value = MagicMock(retun_value="master_key")
                    ret = reactor.list_()
                    assert ret is None

    mock_opts = {}
    mock_opts["reactor"] = [{"test_event/*": ["/srv/reactors/reactor.sls"]}]
    with patch.dict(reactor.__opts__, mock_opts):
        with patch.dict(reactor.__opts__, mock_opts):
            event_returns = {
                "reactors": [{"test_event/*": ["/srv/reactors/reactor.sls"]}],
                "_stamp": "2020-09-04T16:51:52.577711",
            }

        with patch.object(SaltEvent, "connect_pub", return_value=True):
            with patch.object(SaltEvent, "get_event", return_value=event_returns):
                with patch("salt.utils.master.get_master_key") as get_master_key:
                    get_master_key.retun_value = MagicMock(retun_value="master_key")
                    ret = reactor.list_()
                    assert {"test_event/*": ["/srv/reactors/reactor.sls"]} in ret


def test_add():
    """
    test reactor.add runner
    """
    with pytest.raises(CommandExecutionError) as excinfo:
        ret = reactor.add(
            "salt/cloud/*/destroyed", reactors="/srv/reactor/destroy/*.sls"
        )
    assert excinfo.value.error == "Reactor system is not running."

    mock_opts = {}
    mock_opts = {"engines": []}
    with patch.dict(reactor.__opts__, mock_opts):
        with patch.object(SaltEvent, "connect_pub", return_value=True):
            with pytest.raises(CommandExecutionError) as excinfo:
                ret = reactor.add(
                    "salt/cloud/*/destroyed", reactors="/srv/reactor/destroy/*.sls"
                )
            assert excinfo.value.error == "Reactor system is not running."

    mock_opts = {}
    mock_opts["engines"] = [
        {
            "reactor": {
                "refresh_interval": 60,
                "worker_threads": 10,
                "worker_hwm": 10000,
            }
        }
    ]

    event_returns = {
        "reactors": [{"test_event/*": ["/srv/reactors/reactor.sls"]}],
        "result": {"status": True, "comment": "Reactor added."},
        "_stamp": "2020-09-04T17:45:33.206408",
    }

    with patch.dict(reactor.__opts__, mock_opts):
        with patch.object(SaltEvent, "connect_pub", return_value=True):
            with patch.object(SaltEvent, "get_event", return_value=event_returns):
                with patch("salt.utils.master.get_master_key") as get_master_key:
                    get_master_key.retun_value = MagicMock(retun_value="master_key")
                    ret = reactor.add("test_event/*", "/srv/reactor/reactor.sls")
                    assert "status" in ret
                    assert ret["status"]
                    assert "Reactor added." == ret["comment"]

    event_returns = {
        "reactors": [{"test_event/*": ["/srv/reactors/reactor.sls"]}],
        "_stamp": "2020-09-04T17:45:33.206408",
    }

    with patch.dict(reactor.__opts__, mock_opts):
        with patch.object(SaltEvent, "connect_pub", return_value=True):
            with patch.object(SaltEvent, "get_event", return_value=event_returns):
                with patch("salt.utils.master.get_master_key") as get_master_key:
                    get_master_key.retun_value = MagicMock(retun_value="master_key")
                    ret = reactor.add("test_event/*", "/srv/reactor/reactor.sls")
                    assert ret is None


def test_delete():
    """
    test reactor.delete runner
    """
    with pytest.raises(CommandExecutionError) as excinfo:
        ret = reactor.delete("salt/cloud/*/destroyed")
    assert excinfo.value.error == "Reactor system is not running."

    mock_opts = {}
    mock_opts = {"engines": []}
    with patch.dict(reactor.__opts__, mock_opts):
        with pytest.raises(CommandExecutionError) as excinfo:
            ret = reactor.delete("salt/cloud/*/destroyed")
        assert excinfo.value.error == "Reactor system is not running."

    mock_opts = {}
    mock_opts["engines"] = [
        {
            "reactor": {
                "refresh_interval": 60,
                "worker_threads": 10,
                "worker_hwm": 10000,
            }
        }
    ]

    event_returns = {
        "reactors": [{"bot/*": ["/srv/reactors/bot.sls"]}],
        "result": {"status": True, "comment": "Reactor deleted."},
        "_stamp": "2020-09-04T18:15:41.586552",
    }

    with patch.dict(reactor.__opts__, mock_opts):
        with patch.object(SaltEvent, "connect_pub", return_value=True):
            with patch.object(SaltEvent, "get_event", return_value=event_returns):
                with patch("salt.utils.master.get_master_key") as get_master_key:
                    get_master_key.retun_value = MagicMock(retun_value="master_key")
                    ret = reactor.delete("test_event/*")
                    assert "status" in ret
                    assert ret["status"]
                    assert "Reactor deleted." == ret["comment"]

    event_returns = {
        "reactors": [{"bot/*": ["/srv/reactors/bot.sls"]}],
        "_stamp": "2020-09-04T18:15:41.586552",
    }

    with patch.dict(reactor.__opts__, mock_opts):
        with patch.object(SaltEvent, "connect_pub", return_value=True):
            with patch.object(SaltEvent, "get_event", return_value=event_returns):
                with patch("salt.utils.master.get_master_key") as get_master_key:
                    get_master_key.retun_value = MagicMock(retun_value="master_key")
                    ret = reactor.delete("test_event/*")
                    assert ret is None


def test_is_leader():
    """
    test reactor.is_leader runner
    """
    with pytest.raises(CommandExecutionError) as excinfo:
        ret = reactor.is_leader()
    assert excinfo.value.error == "Reactor system is not running."

    mock_opts = {}
    mock_opts = {"engines": []}
    with patch.dict(reactor.__opts__, mock_opts):
        with pytest.raises(CommandExecutionError) as excinfo:
            ret = reactor.is_leader()
        assert excinfo.value.error == "Reactor system is not running."

    mock_opts = {}
    mock_opts["engines"] = [
        {
            "reactor": {
                "refresh_interval": 60,
                "worker_threads": 10,
                "worker_hwm": 10000,
            }
        }
    ]

    event_returns = {"result": True, "_stamp": "2020-09-04T18:32:10.004490"}

    with patch.dict(reactor.__opts__, mock_opts):
        with patch.object(SaltEvent, "connect_pub", return_value=True):
            with patch.object(SaltEvent, "get_event", return_value=event_returns):
                with patch("salt.utils.master.get_master_key") as get_master_key:
                    get_master_key.retun_value = MagicMock(retun_value="master_key")
                    ret = reactor.is_leader()
                    assert ret


def test_set_leader():
    """
    test reactor.set_leader runner
    """
    with pytest.raises(CommandExecutionError) as excinfo:
        ret = reactor.set_leader()
    assert excinfo.value.error == "Reactor system is not running."

    mock_opts = {}
    mock_opts = {"engines": []}
    with patch.dict(reactor.__opts__, mock_opts):
        with pytest.raises(CommandExecutionError) as excinfo:
            ret = reactor.set_leader()
        assert excinfo.value.error == "Reactor system is not running."

    mock_opts = {}
    mock_opts["engines"] = [
        {
            "reactor": {
                "refresh_interval": 60,
                "worker_threads": 10,
                "worker_hwm": 10000,
            }
        }
    ]

    event_returns = {"result": True, "_stamp": "2020-09-04T18:32:10.004490"}

    with patch.dict(reactor.__opts__, mock_opts):
        with patch.object(SaltEvent, "connect_pub", return_value=True):
            with patch.object(SaltEvent, "get_event", return_value=event_returns):
                with patch("salt.utils.master.get_master_key") as get_master_key:
                    get_master_key.retun_value = MagicMock(retun_value="master_key")
                    ret = reactor.set_leader()
                    assert ret



import pytest

import salt.tops.mongo
from tests.support.mock import patch


@pytest.mark.parametrize(
    "expected_ssl, use_ssl",
    [
        (True, {"mongo.ssl": True}),
        (False, {"mongo.ssl": False}),
        (False, {"mongo.ssl": None}),
        (False, {}),
    ],
)
def test_tops_should_correctly_pass_ssl_arg_to_MongoClient(expected_ssl, use_ssl):
    salt.tops.mongo.HAS_PYMONGO = True
    with patch("salt.tops.mongo.pymongo", create=True) as fake_pymongo, patch.dict(
        "salt.tops.mongo.__opts__",
        {
            **use_ssl,
            **{
                "master_tops": {"mongo": {}},
                "mongo.host": "fnord",
                "mongo.port": "fnord",
            },
        },
    ):
        salt.tops.mongo.top(opts={"id": "fnord"})
        fake_pymongo.MongoClient.assert_called_with(
            host="fnord", port="fnord", ssl=expected_ssl
        )


def test_tops_should_pass_auth_to_MongoClient_for_pymongo_v4():
    """
    Test that authentication credentials are passed to MongoClient constructor
    for pymongo v4 compatibility, not using deprecated mdb.authenticate()
    """
    salt.tops.mongo.HAS_PYMONGO = True
    with patch("salt.tops.mongo.pymongo", create=True) as fake_pymongo, patch.dict(
        "salt.tops.mongo.__opts__",
        {
            "master_tops": {"mongo": {}},
            "mongo.host": "localhost",
            "mongo.port": 27017,
            "mongo.user": "testuser",
            "mongo.password": "testpass",
            "mongo.db": "salt",
        },
    ):
        salt.tops.mongo.top(opts={"id": "test-minion"})

        # Verify MongoClient is called with authentication parameters
        fake_pymongo.MongoClient.assert_called_with(
            host="localhost",
            port=27017,
            ssl=False,
            username="testuser",
            password="testpass",
            authSource="admin",
        )

        # Verify that authenticate() is NOT called (pymongo v4 compatibility)
        fake_db = fake_pymongo.MongoClient.return_value.__getitem__.return_value
        assert not fake_db.authenticate.called


def test_tops_should_use_custom_authdb():
    """
    Test that custom authdb is passed correctly to MongoClient
    """
    salt.tops.mongo.HAS_PYMONGO = True
    with patch("salt.tops.mongo.pymongo", create=True) as fake_pymongo, patch.dict(
        "salt.tops.mongo.__opts__",
        {
            "master_tops": {"mongo": {}},
            "mongo.host": "localhost",
            "mongo.port": 27017,
            "mongo.user": "testuser",
            "mongo.password": "testpass",
            "mongo.authdb": "myauthdb",
            "mongo.db": "salt",
        },
    ):
        salt.tops.mongo.top(opts={"id": "test-minion"})

        # Verify custom authSource is used
        fake_pymongo.MongoClient.assert_called_with(
            host="localhost",
            port=27017,
            ssl=False,
            username="testuser",
            password="testpass",
            authSource="myauthdb",
        )


def test_tops_without_auth_should_not_pass_credentials():
    """
    Test that when no credentials are provided, MongoClient is called without auth params
    """
    salt.tops.mongo.HAS_PYMONGO = True
    with patch("salt.tops.mongo.pymongo", create=True) as fake_pymongo, patch.dict(
        "salt.tops.mongo.__opts__",
        {
            "master_tops": {"mongo": {}},
            "mongo.host": "localhost",
            "mongo.port": 27017,
            "mongo.db": "salt",
        },
    ):
        salt.tops.mongo.top(opts={"id": "test-minion"})

        # Verify MongoClient is called without authentication parameters
        fake_pymongo.MongoClient.assert_called_with(
            host="localhost",
            port=27017,
            ssl=False,
        )



import pytest

import salt.runners.net as net
from tests.support.mock import MagicMock

pytestmark = [
    pytest.mark.skipif(
        not net.HAS_NAPALM, reason="napalm module required for this test"
    )
]


@pytest.fixture
def configure_loader_modules(tmp_path):
    mock_get = MagicMock(return_value={})
    return {
        net: {
            "__opts__": {
                "optimization_order": [0, 1, 2],
                "renderer": "yaml",
                "renderer_blacklist": [],
                "renderer_whitelist": [],
                "extension_modules": str(tmp_path),
            },
            "__salt__": {"mine.get": mock_get},
        }
    }


def test_interfaces():
    ret = net.interfaces()
    assert ret is None


def test_findarp():
    ret = net.findarp()
    assert ret is None


def test_findmac():
    ret = net.findmac()
    assert ret is None


def test_lldp():
    ret = net.lldp()
    assert ret is None


def test_find():
    ret = net.find("")
    assert {} == ret


def test_multi_find():
    ret = net.multi_find()
    assert ret is None



"""
Functional tests for the cryptographic primitives the cluster peer
join / discover protocol relies on.

The join handshake in :mod:`salt.channel.server` boils down to::

    # peer B wants to join peer A's cluster
    token  = A.gen_token()                                # random 32 chars
    blob   = A.pub.encrypt(token.encode() + cluster_secret.encode())
    aesblob = A.pub.encrypt(token.encode() + aes_key)
    sig    = B.master_key.sign(packed_payload)

    # peer A receives, decrypts with its own master_rsa, strips token,
    # verifies signature with B's master_pub.

These tests exercise that round-trip against real on-disk keys produced
by :class:`salt.crypt.MasterKeys` so we know the join path stays wired
up correctly when primitive implementations (algorithms, paddings)
change.
"""

import random
import string

import pytest

import salt.crypt
import salt.payload

# The cluster join protocol uses the default RSA OAEP/PKCS1v15 SHA1 algorithms.
# On FIPS-enabled platforms, salt.crypt._enforce_fips rejects SHA1, so these
# tests cannot exercise the protocol until salt/channel/server.py negotiates a
# FIPS-safe algorithm (e.g. OAEP-SHA224 / PKCS1v15-SHA256) for cluster joins.
pytestmark = [pytest.mark.skip_on_fips_enabled_platform]


@pytest.fixture
def cluster_pki_dir(tmp_path):
    path = tmp_path / "cluster_pki"
    path.mkdir()
    (path / "peers").mkdir()
    return path


def _cluster_opts(master_opts, pki_dir, master_id, cluster_pki_dir):
    opts = master_opts.copy()
    opts["id"] = master_id
    opts["pki_dir"] = str(pki_dir)
    opts["cluster_id"] = "master_cluster"
    opts["cluster_pki_dir"] = str(cluster_pki_dir)
    opts["cluster_peers"] = []
    opts["cluster_key_pass"] = None
    opts["cluster_secret"] = None
    opts["key_pass"] = None
    opts["master_sign_pubkey"] = False
    return opts


def _gen_token():
    return "".join(random.choices(string.ascii_letters + string.digits, k=32))


def _build_peer(master_opts, tmp_path, cluster_pki_dir, master_id):
    pki_dir = tmp_path / master_id
    pki_dir.mkdir()
    opts = _cluster_opts(master_opts, pki_dir, master_id, cluster_pki_dir)
    keys = salt.crypt.MasterKeys(opts)
    return opts, keys


def test_join_secret_round_trip(master_opts, tmp_path, cluster_pki_dir):
    """
    Peer B can encrypt ``token + cluster_secret`` under peer A's public
    key, and peer A decrypts with its own private key and recovers the
    original bytes.
    """
    _, keys_a = _build_peer(master_opts, tmp_path, cluster_pki_dir, "127.0.0.1")

    token = _gen_token()
    cluster_secret = "super-secret-cluster-password"

    peer_a_pub_path = cluster_pki_dir / "peers" / "127.0.0.1.pub"
    peer_a_pub = salt.crypt.PublicKey.from_file(peer_a_pub_path)

    blob = peer_a_pub.encrypt(token.encode() + cluster_secret.encode())

    recovered = (
        salt.crypt.PrivateKey.from_file(keys_a.master_rsa_path).decrypt(blob).decode()
    )

    assert recovered.startswith(token)
    assert recovered[len(token) :] == cluster_secret


def test_join_aes_key_round_trip(master_opts, tmp_path, cluster_pki_dir):
    """
    The shared AES session key is handed off the same way the secret
    is: ``token + aes_key`` encrypted under the target peer's pub.
    Peer A recovering the AES blob must get the original bytes back
    exactly, with no text decoding in the middle -- the AES key is raw
    binary.
    """
    _, keys_a = _build_peer(master_opts, tmp_path, cluster_pki_dir, "127.0.0.1")

    token = _gen_token()
    aes_key = salt.crypt.Crypticle.generate_key_string().encode()

    peer_a_pub = salt.crypt.PublicKey.from_file(
        cluster_pki_dir / "peers" / "127.0.0.1.pub"
    )

    blob = peer_a_pub.encrypt(token.encode() + aes_key)

    recovered = salt.crypt.PrivateKey.from_file(keys_a.master_rsa_path).decrypt(blob)

    assert recovered.startswith(token.encode())
    assert recovered[len(token) :] == aes_key


def test_join_signature_verifies_across_peers(master_opts, tmp_path, cluster_pki_dir):
    """
    Peer B signs the packed join payload with its own master private
    key. Peer A verifies the signature using peer B's pub key from the
    shared ``peers/`` directory.
    """
    _, keys_b = _build_peer(master_opts, tmp_path, cluster_pki_dir, "127.0.0.2")
    _build_peer(master_opts, tmp_path, cluster_pki_dir, "127.0.0.1")

    payload = salt.payload.package(
        {
            "peer_id": "127.0.0.2",
            "token": _gen_token(),
            "pub": (cluster_pki_dir / "peers" / "127.0.0.2.pub").read_text(
                encoding="utf-8"
            ),
        }
    )
    sig = salt.crypt.PrivateKey.from_file(keys_b.master_rsa_path).sign(payload)

    peer_b_pub = salt.crypt.PublicKey.from_file(
        cluster_pki_dir / "peers" / "127.0.0.2.pub"
    )
    assert peer_b_pub.verify(payload, sig) is True


def test_join_signature_rejects_tampered_payload(
    master_opts, tmp_path, cluster_pki_dir
):
    """
    Any modification of the signed payload between peers must be
    detected -- otherwise an attacker could replace the pub key inside
    a legitimate join frame.
    """
    _, keys_b = _build_peer(master_opts, tmp_path, cluster_pki_dir, "127.0.0.2")

    payload = salt.payload.package({"peer_id": "127.0.0.2", "token": "abc"})
    sig = salt.crypt.PrivateKey.from_file(keys_b.master_rsa_path).sign(payload)

    tampered = payload + b"\x00"
    peer_b_pub = salt.crypt.PublicKey.from_file(
        cluster_pki_dir / "peers" / "127.0.0.2.pub"
    )
    assert peer_b_pub.verify(tampered, sig) is False


def test_join_signature_rejects_wrong_signer(master_opts, tmp_path, cluster_pki_dir):
    """
    A payload signed by peer C must NOT verify when the receiver looks
    up the claimed sender (peer B) in ``peers/`` -- the join handshake
    leans on this so an authorised peer cannot impersonate another.
    """
    _build_peer(master_opts, tmp_path, cluster_pki_dir, "127.0.0.2")
    _, keys_c = _build_peer(master_opts, tmp_path, cluster_pki_dir, "127.0.0.3")

    payload = salt.payload.package({"peer_id": "127.0.0.2", "token": "abc"})
    sig = salt.crypt.PrivateKey.from_file(keys_c.master_rsa_path).sign(payload)

    peer_b_pub = salt.crypt.PublicKey.from_file(
        cluster_pki_dir / "peers" / "127.0.0.2.pub"
    )
    assert peer_b_pub.verify(payload, sig) is False



import salt.executors.sudo
from tests.support.mock import MagicMock, patch


def test_sudo_execute_adds_priv_arg():
    # Setup inputs
    opts = {"sudo_user": "saltdev", "config_dir": "/etc/salt"}
    data = {"fun": "test.ping"}
    func = MagicMock()
    args = []
    kwargs = {}

    # Mock __salt__ and cmd.run_all
    mock_run_all = MagicMock(
        return_value={
            "retcode": 0,
            "stdout": '{"local": {"return": true, "retcode": 0}}',
        }
    )

    # Mock __context__
    context_dict = {}

    # Initialize dunder dictionaries if they don't exist
    if not hasattr(salt.executors.sudo, "__salt__"):
        salt.executors.sudo.__salt__ = {}
    if not hasattr(salt.executors.sudo, "__context__"):
        salt.executors.sudo.__context__ = {}

    with patch.dict(
        salt.executors.sudo.__salt__, {"cmd.run_all": mock_run_all}
    ), patch.dict(salt.executors.sudo.__context__, context_dict):

        salt.executors.sudo.execute(opts, data, func, args, kwargs)

        # Verify the command called includes --priv saltdev
        call_args = mock_run_all.call_args[0][0]

        # Check expected parts of command
        assert "sudo" in call_args
        assert "-u" in call_args
        assert "saltdev" in call_args
        assert "salt-call" in call_args
        assert "--priv" in call_args

        # Verify --priv follows salt-call and precedes saltdev
        salt_call_idx = call_args.index("salt-call")
        priv_idx = call_args.index("--priv")
        user_idx = priv_idx + 1

        assert priv_idx > salt_call_idx
        assert call_args[user_idx] == "saltdev"



"""
    Test cases for salt.modules.reg
"""

import pytest
from saltfactories.utils import random_string

import salt.modules.reg as reg
import salt.utils.stringutils
import salt.utils.win_reg
from salt.exceptions import CommandExecutionError
from tests.support.mock import MagicMock, patch

try:
    import win32api

    HAS_WIN32 = True
except ImportError:
    HAS_WIN32 = False

pytestmark = [
    pytest.mark.windows_whitelisted,
    pytest.mark.skip_unless_on_windows,
    pytest.mark.destructive_test,
    pytest.mark.skipif(HAS_WIN32 is False, reason="Tests require win32 libraries"),
]


@pytest.fixture
def unicode_key():
    return "Unicode Key \N{TRADE MARK SIGN}"


@pytest.fixture
def unicode_value():
    return "Unicode Value \N{COPYRIGHT SIGN},\N{TRADE MARK SIGN},\N{REGISTERED SIGN}"


@pytest.fixture
def fake_key():
    return "SOFTWARE\\{}".format(random_string("SaltTesting-", lowercase=False))


@pytest.fixture
def configure_loader_modules():
    return {
        reg: {
            "__utils__": {
                "reg.delete_value": salt.utils.win_reg.delete_value,
                "reg.delete_key_recursive": salt.utils.win_reg.delete_key_recursive,
                "reg.key_exists": salt.utils.win_reg.key_exists,
                "reg.list_keys": salt.utils.win_reg.list_keys,
                "reg.list_values": salt.utils.win_reg.list_values,
                "reg.read_value": salt.utils.win_reg.read_value,
                "reg.set_value": salt.utils.win_reg.set_value,
                "reg.value_exists": salt.utils.win_reg.value_exists,
            }
        }
    }


def test_key_exists_existing():
    """
    Tests the key_exists function using a well known registry key
    """
    assert reg.key_exists(hive="HKLM", key="SOFTWARE\\Microsoft")


def test_key_exists_non_existing(fake_key):
    """
    Tests the key_exists function using a non existing registry key
    """
    assert not reg.key_exists(hive="HKLM", key=fake_key)


def test_key_exists_invalid_hive():
    """
    Tests the key_exists function using an invalid hive
    """
    with pytest.raises(CommandExecutionError):
        reg.key_exists(hive="BADHIVE", key="SOFTWARE\\Microsoft")


def test_key_exists_unknown_key_error():
    """
    Tests the key_exists function with an unknown key error
    """
    mock_error = MagicMock(
        side_effect=win32api.error(123, "RegOpenKeyEx", "Unknown error")
    )
    with patch("salt.utils.win_reg.win32api.RegOpenKeyEx", mock_error):
        with pytest.raises(win32api.error):
            reg.key_exists(hive="HKLM", key="SOFTWARE\\Microsoft")


def test_value_exists_existing():
    """
    Tests the value_exists function using a well known registry key
    """
    result = reg.value_exists(
        hive="HKLM",
        key="SOFTWARE\\Microsoft\\Windows\\CurrentVersion",
        vname="CommonFilesDir",
    )
    assert result


def test_value_exists_non_existing():
    """
    Tests the value_exists function using a non existing registry key
    """
    result = reg.value_exists(
        hive="HKLM",
        key="SOFTWARE\\Microsoft\\Windows\\CurrentVersion",
        vname="NonExistingValueName",
    )
    assert not result


def test_value_exists_invalid_hive():
    """
    Tests the value_exists function using an invalid hive
    """
    with pytest.raises(CommandExecutionError):
        reg.value_exists(
            hive="BADHIVE",
            key="SOFTWARE\\Microsoft\\Windows\\CurrentVersion",
            vname="CommonFilesDir",
        )


def test_value_exists_key_not_exist():
    """
    Tests the value_exists function when the key does not exist
    """
    mock_error = MagicMock(
        side_effect=win32api.error(2, "RegOpenKeyEx", "Unknown error")
    )
    with patch("salt.utils.win_reg.win32api.RegOpenKeyEx", mock_error):
        result = reg.value_exists(
            hive="HKLM",
            key="SOFTWARE\\Microsoft\\Windows\\CurrentVersion",
            vname="CommonFilesDir",
        )
    assert not result


def test_value_exists_unknown_key_error():
    """
    Tests the value_exists function with an unknown error when opening the
    key
    """
    mock_error = MagicMock(
        side_effect=win32api.error(123, "RegOpenKeyEx", "Unknown error")
    )
    with patch("salt.utils.win_reg.win32api.RegOpenKeyEx", mock_error):
        with pytest.raises(win32api.error):
            reg.value_exists(
                hive="HKLM",
                key="SOFTWARE\\Microsoft\\Windows\\CurrentVersion",
                vname="CommonFilesDir",
            )


def test_value_exists_empty_default_value():
    """
    Tests the value_exists function when querying the default value
    """
    mock_error = MagicMock(
        side_effect=win32api.error(2, "RegQueryValueEx", "Empty Value")
    )
    with patch("salt.utils.win_reg.win32api.RegQueryValueEx", mock_error):
        result = reg.value_exists(
            hive="HKLM",
            key="SOFTWARE\\Microsoft\\Windows\\CurrentVersion",
            vname=None,
        )
    assert result


def test_value_exists_no_vname():
    """
    Tests the value_exists function when the vname does not exist
    """
    mock_error = MagicMock(
        side_effect=win32api.error(123, "RegQueryValueEx", "Empty Value")
    )
    with patch("salt.utils.win_reg.win32api.RegQueryValueEx", mock_error):
        result = reg.value_exists(
            hive="HKLM",
            key="SOFTWARE\\Microsoft\\Windows\\CurrentVersion",
            vname="NonExistingValuePair",
        )
    assert not result


def test_list_keys_existing():
    """
    Test the list_keys function using a well known registry key
    """
    assert "Microsoft" in reg.list_keys(hive="HKLM", key="SOFTWARE")


def test_list_keys_non_existing(fake_key):
    """
    Test the list_keys function using a non existing registry key
    """
    expected = (False, f"Cannot find key: HKLM\\{fake_key}")
    assert reg.list_keys(hive="HKLM", key=fake_key) == expected


def test_list_keys_invalid_hive():
    """
    Test the list_keys function when passing an invalid hive
    """
    with pytest.raises(CommandExecutionError):
        reg.list_keys(hive="BADHIVE", key="SOFTWARE\\Microsoft")


def test_list_keys_unknown_key_error():
    """
    Tests the list_keys function with an unknown key error
    """
    mock_error = MagicMock(
        side_effect=win32api.error(123, "RegOpenKeyEx", "Unknown error")
    )
    with patch("salt.utils.win_reg.win32api.RegOpenKeyEx", mock_error):
        with pytest.raises(win32api.error):
            reg.list_keys(hive="HKLM", key="SOFTWARE\\Microsoft")


def test_list_values_existing():
    """
    Test the list_values function using a well known registry key
    """
    values = reg.list_values(
        hive="HKLM", key="SOFTWARE\\Microsoft\\Windows\\CurrentVersion"
    )
    keys = []
    for value in values:
        keys.append(value["vname"])
    assert "ProgramFilesDir" in keys


def test_list_values_non_existing(fake_key):
    """
    Test the list_values function using a non existing registry key
    """
    expected = (False, f"Cannot find key: HKLM\\{fake_key}")
    assert reg.list_values(hive="HKLM", key=fake_key) == expected


def test_list_values_invalid_hive():
    """
    Test the list_values function when passing an invalid hive
    """
    with pytest.raises(CommandExecutionError):
        reg.list_values(hive="BADHIVE", key="SOFTWARE\\Microsoft")


def test_list_values_unknown_key_error():
    """
    Tests the list_values function with an unknown key error
    """
    mock_error = MagicMock(
        side_effect=win32api.error(123, "RegOpenKeyEx", "Unknown error")
    )
    with patch("salt.utils.win_reg.win32api.RegOpenKeyEx", mock_error):
        with pytest.raises(win32api.error):
            reg.list_values(hive="HKLM", key="SOFTWARE\\Microsoft")


def test_read_value_existing():
    """
    Test the read_value function using a well known registry value
    """
    ret = reg.read_value(
        hive="HKLM",
        key="SOFTWARE\\Microsoft\\Windows\\CurrentVersion",
        vname="ProgramFilesPath",
    )
    assert ret["vdata"] == "%ProgramFiles%"


def test_read_value_default():
    """
    Test the read_value function reading the default value using a well
    known registry key
    """
    ret = reg.read_value(
        hive="HKLM", key="SOFTWARE\\Microsoft\\Windows\\CurrentVersion"
    )
    assert ret["vdata"] == "(value not set)"


def test_read_value_non_existing():
    """
    Test the read_value function using a non existing value pair
    """
    expected = {
        "comment": (
            "Cannot find fake_name in HKLM\\SOFTWARE\\Microsoft\\"
            "Windows\\CurrentVersion"
        ),
        "vdata": None,
        "vtype": None,
        "vname": "fake_name",
        "success": False,
        "hive": "HKLM",
        "key": "SOFTWARE\\Microsoft\\Windows\\CurrentVersion",
    }
    result = reg.read_value(
        hive="HKLM",
        key="SOFTWARE\\Microsoft\\Windows\\CurrentVersion",
        vname="fake_name",
    )
    assert result == expected


def test_read_value_non_existing_key(fake_key):
    """
    Test the read_value function using a non existing registry key
    """
    expected = {
        "comment": f"Cannot find key: HKLM\\{fake_key}",
        "vdata": None,
        "vtype": None,
        "vname": "fake_name",
        "success": False,
        "hive": "HKLM",
        "key": fake_key,
    }
    result = reg.read_value(hive="HKLM", key=fake_key, vname="fake_name")
    assert result == expected


def test_read_value_invalid_hive():
    """
    Test the read_value function when passing an invalid hive
    """
    with pytest.raises(CommandExecutionError):
        reg.read_value(
            hive="BADHIVE",
            key="SOFTWARE\\Microsoft",
            vname="ProgramFilesPath",
        )


def test_read_value_unknown_key_error():
    """
    Tests the read_value function with an unknown key error
    """
    mock_error = MagicMock(
        side_effect=win32api.error(123, "RegOpenKeyEx", "Unknown error")
    )
    with patch("salt.utils.win_reg.win32api.RegOpenKeyEx", mock_error):
        with pytest.raises(win32api.error):
            reg.read_value(
                hive="HKLM",
                key="SOFTWARE\\Microsoft\\Windows\\CurrentVersion",
                vname="ProgramFilesPath",
            )


def test_read_value_unknown_value_error():
    """
    Tests the read_value function with an unknown value error
    """
    mock_error = MagicMock(
        side_effect=win32api.error(123, "RegQueryValueEx", "Unknown error")
    )
    with patch("salt.utils.win_reg.win32api.RegQueryValueEx", mock_error):
        with pytest.raises(win32api.error):
            reg.read_value(
                hive="HKLM",
                key="SOFTWARE\\Microsoft\\Windows\\CurrentVersion",
                vname="ProgramFilesPath",
            )


@pytest.mark.destructive_test
def test_read_value_multi_sz_empty_list(fake_key):
    """
    An empty REG_MULTI_SZ value should return an empty list, not None
    """
    try:
        assert reg.set_value(
            hive="HKLM",
            key=fake_key,
            vname="empty_list",
            vdata=[],
            vtype="REG_MULTI_SZ",
        )
        expected = {
            "hive": "HKLM",
            "key": fake_key,
            "success": True,
            "vdata": [],
            "vname": "empty_list",
            "vtype": "REG_MULTI_SZ",
        }
        result = reg.read_value(hive="HKLM", key=fake_key, vname="empty_list")
        assert result == expected
    finally:
        reg.delete_key_recursive(hive="HKLM", key=fake_key)


@pytest.mark.destructive_test
def test_set_value(fake_key):
    """
    Test the set_value function
    """
    try:
        assert reg.set_value(
            hive="HKLM", key=fake_key, vname="fake_name", vdata="fake_data"
        )
        expected = {
            "hive": "HKLM",
            "key": fake_key,
            "success": True,
            "vdata": "fake_data",
            "vname": "fake_name",
            "vtype": "REG_SZ",
        }
        result = reg.read_value(hive="HKLM", key=fake_key, vname="fake_name")
        assert result == expected
    finally:
        reg.delete_key_recursive(hive="HKLM", key=fake_key)


@pytest.mark.destructive_test
def test_set_value_default(fake_key):
    """
    Test the set_value function on the default value
    """
    try:
        assert reg.set_value(hive="HKLM", key=fake_key, vdata="fake_default_data")
        expected = {
            "hive": "HKLM",
            "key": fake_key,
            "success": True,
            "vdata": "fake_default_data",
            "vname": "(Default)",
            "vtype": "REG_SZ",
        }
        result = reg.read_value(hive="HKLM", key=fake_key)
        assert result == expected
    finally:
        reg.delete_key_recursive(hive="HKLM", key=fake_key)


@pytest.mark.destructive_test
def test_set_value_unicode_key(fake_key, unicode_key):
    """
    Test the set_value function on a unicode key
    """
    try:
        assert reg.set_value(
            hive="HKLM",
            key="\\".join([fake_key, unicode_key]),
            vname="fake_name",
            vdata="fake_value",
        )
        expected = {
            "hive": "HKLM",
            "key": "\\".join([fake_key, unicode_key]),
            "success": True,
            "vdata": "fake_value",
            "vname": "fake_name",
            "vtype": "REG_SZ",
        }
        result = reg.read_value(
            hive="HKLM",
            key="\\".join([fake_key, unicode_key]),
            vname="fake_name",
        )
        assert result == expected
    finally:
        reg.delete_key_recursive(hive="HKLM", key=fake_key)


@pytest.mark.destructive_test
def test_set_value_unicode_value(fake_key, unicode_value):
    """
    Test the set_value function on a unicode value
    """
    try:
        assert reg.set_value(
            hive="HKLM", key=fake_key, vname="fake_unicode", vdata=unicode_value
        )
        expected = {
            "hive": "HKLM",
            "key": fake_key,
            "success": True,
            "vdata": unicode_value,
            "vname": "fake_unicode",
            "vtype": "REG_SZ",
        }
        result = reg.read_value(hive="HKLM", key=fake_key, vname="fake_unicode")
        assert result == expected
    finally:
        reg.delete_key_recursive(hive="HKLM", key=fake_key)


@pytest.mark.destructive_test
def test_set_value_reg_dword(fake_key):
    """
    Test the set_value function on a REG_DWORD value
    """
    try:
        assert reg.set_value(
            hive="HKLM",
            key=fake_key,
            vname="dword_value",
            vdata=123,
            vtype="REG_DWORD",
        )
        expected = {
            "hive": "HKLM",
            "key": fake_key,
            "success": True,
            "vdata": 123,
            "vname": "dword_value",
            "vtype": "REG_DWORD",
        }
        result = reg.read_value(hive="HKLM", key=fake_key, vname="dword_value")
        assert result == expected
    finally:
        reg.delete_key_recursive(hive="HKLM", key=fake_key)


@pytest.mark.destructive_test
def test_set_value_reg_qword(fake_key):
    """
    Test the set_value function on a REG_QWORD value
    """
    try:
        assert reg.set_value(
            hive="HKLM",
            key=fake_key,
            vname="qword_value",
            vdata=123,
            vtype="REG_QWORD",
        )
        expected = {
            "hive": "HKLM",
            "key": fake_key,
            "success": True,
            "vdata": 123,
            "vname": "qword_value",
            "vtype": "REG_QWORD",
        }
        result = reg.read_value(hive="HKLM", key=fake_key, vname="qword_value")
        assert result == expected
    finally:
        reg.delete_key_recursive(hive="HKLM", key=fake_key)


def test_set_value_invalid_hive(fake_key):
    """
    Test the set_value function when passing an invalid hive
    """
    with pytest.raises(CommandExecutionError):
        reg.set_value(
            hive="BADHIVE",
            key=fake_key,
            vname="fake_name",
            vdata="fake_data",
        )


def test_set_value_open_create_failure(fake_key):
    """
    Test the set_value function when there is a problem opening/creating
    the key
    """
    mock_error = MagicMock(
        side_effect=win32api.error(123, "RegCreateKeyEx", "Unknown error")
    )
    with patch("salt.utils.win_reg.win32api.RegCreateKeyEx", mock_error):
        result = reg.set_value(
            hive="HKLM", key=fake_key, vname="fake_name", vdata="fake_data"
        )
    assert not result


def test_set_value_type_error(fake_key):
    """
    Test the set_value function when the wrong type of data is passed
    """
    mock_error = MagicMock(side_effect=TypeError("Mocked TypeError"))
    with patch("salt.utils.win_reg.win32api.RegSetValueEx", mock_error):
        assert not reg.set_value(
            hive="HKLM", key=fake_key, vname="fake_name", vdata="fake_data"
        )


def test_set_value_system_error(fake_key):
    """
    Test the set_value function when a SystemError occurs while setting the
    value
    """
    mock_error = MagicMock(side_effect=SystemError("Mocked SystemError"))
    with patch("salt.utils.win_reg.win32api.RegSetValueEx", mock_error):
        assert not reg.set_value(
            hive="HKLM", key=fake_key, vname="fake_name", vdata="fake_data"
        )


def test_set_value_value_error(fake_key):
    """
    Test the set_value function when a ValueError occurs while setting the
    value
    """
    mock_error = MagicMock(side_effect=ValueError("Mocked ValueError"))
    with patch("salt.utils.win_reg.win32api.RegSetValueEx", mock_error):
        assert not reg.set_value(
            hive="HKLM", key=fake_key, vname="fake_name", vdata="fake_data"
        )


@pytest.mark.destructive_test
def test_delete_value(fake_key):
    """
    Test the delete_value function
    """
    try:
        assert reg.set_value(
            hive="HKLM", key=fake_key, vname="fake_name", vdata="fake_data"
        )
        assert reg.delete_value(hive="HKLM", key=fake_key, vname="fake_name")
    finally:
        reg.delete_key_recursive(hive="HKLM", key=fake_key)


def test_delete_value_non_existing(fake_key):
    """
    Test the delete_value function on non existing value
    """
    mock_error = MagicMock(
        side_effect=win32api.error(2, "RegOpenKeyEx", "Unknown error")
    )
    with patch("salt.utils.win_reg.win32api.RegOpenKeyEx", mock_error):
        result = reg.delete_value(hive="HKLM", key=fake_key, vname="fake_name")
    assert result is None


def test_delete_value_invalid_hive(fake_key):
    """
    Test the delete_value function when passing an invalid hive
    """
    with pytest.raises(CommandExecutionError):
        reg.delete_value(hive="BADHIVE", key=fake_key, vname="fake_name")


def test_delete_value_unknown_error(fake_key):
    """
    Test the delete_value function when there is a problem opening the key
    """
    mock_error = MagicMock(
        side_effect=win32api.error(123, "RegOpenKeyEx", "Unknown error")
    )
    with patch("salt.utils.win_reg.win32api.RegOpenKeyEx", mock_error):
        with pytest.raises(win32api.error):
            reg.delete_value(
                hive="HKLM",
                key=fake_key,
                vname="fake_name",
            )


@pytest.mark.destructive_test
def test_delete_value_unicode(fake_key, unicode_value):
    """
    Test the delete_value function on a unicode value
    """
    try:
        assert reg.set_value(
            hive="HKLM", key=fake_key, vname="fake_unicode", vdata=unicode_value
        )
        assert reg.delete_value(hive="HKLM", key=fake_key, vname="fake_unicode")
    finally:
        reg.delete_key_recursive(hive="HKLM", key=fake_key)


@pytest.mark.destructive_test
def test_delete_value_unicode_vname(fake_key, unicode_key):
    """
    Test the delete_value function on a unicode vname
    """
    try:
        assert reg.set_value(
            hive="HKLM", key=fake_key, vname=unicode_key, vdata="junk data"
        )
        assert reg.delete_value(hive="HKLM", key=fake_key, vname=unicode_key)
    finally:
        reg.delete_key_recursive(hive="HKLM", key=fake_key)


@pytest.mark.destructive_test
def test_delete_value_unicode_key(fake_key, unicode_key, unicode_value):
    """
    Test the delete_value function on a unicode key
    """
    try:
        assert reg.set_value(
            hive="HKLM",
            key="\\".join([fake_key, unicode_key]),
            vname="fake_name",
            vdata="junk data",
        )
        assert reg.delete_value(
            hive="HKLM",
            key="\\".join([fake_key, unicode_key]),
            vname="fake_name",
        )
    finally:
        reg.delete_key_recursive(hive="HKLM", key=fake_key)


def test_delete_key_recursive_invalid_hive(fake_key):
    """
    Test the delete_key_recursive function when passing an invalid hive
    """
    with pytest.raises(CommandExecutionError):
        reg.delete_key_recursive(hive="BADHIVE", key=fake_key)


def test_delete_key_recursive_key_not_found(fake_key):
    """
    Test the delete_key_recursive function when the passed key to delete is
    not found.
    """
    assert not reg.key_exists(hive="HKLM", key=fake_key)
    assert not reg.delete_key_recursive(hive="HKLM", key=fake_key)


def test_delete_key_recursive_too_close():
    """
    Test the delete_key_recursive function when the passed key to delete is
    too close to root, such as
    """
    mock_true = MagicMock(return_value=True)
    with patch("salt.utils.win_reg.key_exists", mock_true):
        assert not reg.delete_key_recursive(hive="HKLM", key="fake_key")


@pytest.mark.destructive_test
def test_delete_key_recursive(fake_key):
    """
    Test the delete_key_recursive function
    """
    try:
        assert reg.set_value(
            hive="HKLM", key=fake_key, vname="fake_name", vdata="fake_value"
        )
        expected = {"Deleted": ["\\".join(["HKLM", fake_key])], "Failed": []}
        result = reg.delete_key_recursive(hive="HKLM", key=fake_key)
        assert result == expected
    finally:
        reg.delete_key_recursive(hive="HKLM", key=fake_key)


@pytest.mark.destructive_test
def test_delete_key_recursive_failed_to_open_key(fake_key):
    """
    Test the delete_key_recursive function on failure to open the key
    """
    try:
        assert reg.set_value(
            hive="HKLM", key=fake_key, vname="fake_name", vdata="fake_value"
        )
        expected = {
            "Deleted": [],
            "Failed": ["\\".join(["HKLM", fake_key]) + " Failed to connect to key"],
        }
        mock_true = MagicMock(return_value=True)
        mock_error = MagicMock(
            side_effect=[
                1,
                win32api.error(3, "RegOpenKeyEx", "Failed to connect to key"),
            ]
        )
        with patch("salt.utils.win_reg.key_exists", mock_true), patch(
            "salt.utils.win_reg.win32api.RegOpenKeyEx", mock_error
        ):
            result = reg.delete_key_recursive(hive="HKLM", key=fake_key)
        assert result == expected
    finally:
        reg.delete_key_recursive(hive="HKLM", key=fake_key)


@pytest.mark.destructive_test
def test_delete_key_recursive_failed_to_delete(fake_key):
    """
    Test the delete_key_recursive function on failure to delete a key
    """
    try:
        assert reg.set_value(
            hive="HKLM", key=fake_key, vname="fake_name", vdata="fake_value"
        )
        expected = {
            "Deleted": [],
            "Failed": ["\\".join(["HKLM", fake_key]) + " Unknown error"],
        }
        # pylint: disable=undefined-variable
        mock_error = MagicMock(side_effect=WindowsError("Unknown error"))
        # pylint: enable=undefined-variable
        with patch("salt.utils.win_reg.win32api.RegDeleteKey", mock_error):
            result = reg.delete_key_recursive(hive="HKLM", key=fake_key)
        assert result == expected
    finally:
        reg.delete_key_recursive(hive="HKLM", key=fake_key)


@pytest.mark.destructive_test
def test_delete_key_recursive_unicode(fake_key, unicode_key):
    """
    Test the delete_key_recursive function on value within a unicode key
    """
    try:
        assert reg.set_value(
            hive="HKLM",
            key="\\".join([fake_key, unicode_key]),
            vname="fake_name",
            vdata="fake_value",
        )
        expected = {
            "Deleted": ["\\".join(["HKLM", fake_key, unicode_key])],
            "Failed": [],
        }
        result = reg.delete_key_recursive(
            hive="HKLM", key="\\".join([fake_key, unicode_key])
        )
        assert result == expected
    finally:
        reg.delete_key_recursive(hive="HKLM", key=fake_key)



"""
tests for host state
"""

import logging
import os
import shutil

import pytest

import salt.utils.files
import salt.utils.stringutils
from tests.support.case import ModuleCase
from tests.support.mixins import SaltReturnAssertsMixin
from tests.support.runtests import RUNTIME_VARS

log = logging.getLogger(__name__)


@pytest.mark.windows_whitelisted
class HostTest(ModuleCase, SaltReturnAssertsMixin):
    """
    Validate the host state
    """

    @classmethod
    def setUpClass(cls):
        cls.hosts_file = os.path.join(RUNTIME_VARS.TMP, "hosts")

    def __clear_hosts(self):
        """
        Delete the tmp hosts file
        """
        if os.path.isfile(self.hosts_file):
            os.remove(self.hosts_file)

    def setUp(self):
        shutil.copyfile(os.path.join(RUNTIME_VARS.FILES, "hosts"), self.hosts_file)
        self.addCleanup(self.__clear_hosts)
        super().setUp()

    @pytest.mark.slow_test
    def test_present(self):
        """
        host.present
        """
        name = "spam.bacon"
        ip = "10.10.10.10"
        ret = self.run_state("host.present", name=name, ip=ip)
        self.assertSaltTrueReturn(ret)
        with salt.utils.files.fopen(self.hosts_file) as fp_:
            output = salt.utils.stringutils.to_unicode(fp_.read())
            self.assertIn(f"{ip}\t\t{name}", output)



import pytest

import salt.auth

pytestmark = [
    pytest.mark.slow_test,
    pytest.mark.skip_if_not_root,
    pytest.mark.destructive_test,
    pytest.mark.windows_whitelisted,
]


def test_master_call(client, auth_creds, salt_auto_account):
    """
    Test executing master_call with lowdata

    The choice of using key.list_all for this is arbitrary and should be
    changed to some mocked function that is more testing friendly.
    """
    low = {"client": "wheel", "fun": "key.list_all", "print_event": False, **auth_creds}
    ret = client.master_call(**low)
    assert ret
    assert "data" in ret
    data = ret["data"]
    assert data["success"] is True
    assert data["user"] == salt_auto_account.username
    assert data["fun"] == "wheel.key.list_all"
    assert data["return"]
    master_id = client.opts["id"].removesuffix("_master")
    assert data["return"]["local"] == [f"{master_id}.pem", f"{master_id}.pub"]


def test_token(client, client_config, auth_creds, salt_auto_account):
    """
    Test executing master_call with lowdata

    The choice of using key.list_all for this is arbitrary and should be
    changed to some mocked function that is more testing friendly.
    """
    auth = salt.auth.LoadAuth(client_config)
    token = auth.mk_token(auth_creds)

    ret = client.master_call(
        **{
            "client": "wheel",
            "fun": "key.list_all",
            "token": token["token"],
            "print_event": False,
        }
    )
    assert ret
    assert "data" in ret
    data = ret["data"]
    assert data["success"] is True
    assert data["user"] == salt_auto_account.username
    assert data["fun"] == "wheel.key.list_all"
    assert data["return"]
    master_id = client.opts["id"].removesuffix("_master")
    assert data["return"]["local"] == [f"{master_id}.pem", f"{master_id}.pub"]


def test_cmd_sync(client, auth_creds, salt_auto_account):
    low = {"client": "async", "fun": "key.list_all", "print_event": False, **auth_creds}

    ret = client.cmd_sync(low)
    assert ret
    assert "data" in ret
    data = ret["data"]
    assert data["success"] is True
    assert data["user"] == salt_auto_account.username
    assert data["fun"] == "wheel.key.list_all"
    assert data["return"]
    master_id = client.opts["id"].removesuffix("_master")
    assert data["return"]["local"] == [f"{master_id}.pem", f"{master_id}.pub"]


# Remove this skipIf when https://github.com/saltstack/salt/issues/39616 is resolved
@pytest.mark.skip_on_windows(reason="Causes pickling error on Windows: Issue #39616")
def test_cmd_async(client, auth_creds):
    low = {
        "client": "wheel_async",
        "fun": "key.list_all",
        "print_event": False,
        **auth_creds,
    }

    ret = client.cmd_async(low)
    # Being an async command, we won't get any data back, just the JID and the TAG
    assert ret
    assert "jid" in ret
    assert "tag" in ret
    assert ret["tag"] == "salt/wheel/{}".format(ret["jid"])


def test_cmd_sync_w_arg(client, auth_creds, salt_auto_account):
    low = {"fun": "key.finger", "match": "*", "print_event": False, **auth_creds}
    ret = client.cmd_sync(low)
    assert ret
    assert "data" in ret
    data = ret["data"]
    assert data["success"] is True
    assert data["user"] == salt_auto_account.username
    assert data["fun"] == "wheel.key.finger"
    assert data["return"]
    assert data["return"]["local"]
    master_id = client.opts["id"].removesuffix("_master")
    assert f"{master_id}.pem" in data["return"]["local"]
    assert f"{master_id}.pub" in data["return"]["local"]


def test_wildcard_auth(client):
    # This test passes because of the following master config
    # external_auth:
    #   auto:
    #     '*':
    #       - '@wheel'
    #       - '@runner'
    #       - test.*
    username = "the_s0und_of_t3ch"
    low = {
        "username": username,
        "password": "willrockyou",
        "eauth": "auto",
        "fun": "key.list_all",
        "print_event": False,
    }
    ret = client.cmd_sync(low)
    assert ret
    assert "data" in ret
    data = ret["data"]
    assert data["success"] is True
    assert data["user"] == username
    assert data["fun"] == "wheel.key.list_all"
    assert data["return"]
    master_id = client.opts["id"].removesuffix("_master")
    assert data["return"]["local"] == [f"{master_id}.pem", f"{master_id}.pub"]



"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""

import pytest

import salt.modules.data as data
from tests.support.mock import MagicMock, mock_open, patch


@pytest.fixture
def configure_loader_modules():
    return {data: {}}


# 'clear' function tests: 1


def test_clear():
    """
    Test if it clear out all of the data in the minion datastore
    """
    with patch("os.remove", MagicMock(return_value="")):
        with patch.dict(data.__opts__, {"cachedir": ""}):
            assert data.clear()


# 'load' function tests: 1


def test_load():
    """
    Test if it return all of the data in the minion datastore
    """
    mocked_fopen = MagicMock(return_value=True)
    mocked_fopen.__enter__ = MagicMock(return_value=mocked_fopen)
    mocked_fopen.__exit__ = MagicMock()
    with patch("salt.utils.files.fopen", MagicMock(return_value=mocked_fopen)):
        with patch("salt.payload.loads", MagicMock(return_value=True)):
            with patch.dict(data.__opts__, {"cachedir": "/"}):
                assert data.load()


# 'dump' function tests: 3


def test_dump():
    """
    Test if it replace the entire datastore with a passed data structure
    """
    with patch.dict(data.__opts__, {"cachedir": "/"}):
        with patch("salt.utils.files.fopen", mock_open()):
            assert data.dump('{"eggs": "spam"}')


def test_dump_isinstance():
    """
    Test if it replace the entire datastore with a passed data structure
    """
    with patch("ast.literal_eval", MagicMock(return_value="")):
        assert not data.dump("salt")


def test_dump_ioerror():
    """
    Test if it replace the entire datastore with a passed data structure
    """
    with patch.dict(data.__opts__, {"cachedir": "/"}):
        mock = MagicMock(side_effect=IOError(""))
        with patch("salt.utils.files.fopen", mock):
            assert not data.dump('{"eggs": "spam"}')


# 'update' function tests: 1


def test_update():
    """
    Test if it update a key with a value in the minion datastore
    """
    with patch("salt.modules.data.load", MagicMock(return_value={})), patch(
        "salt.modules.data.dump", MagicMock(return_value=True)
    ):
        assert data.update("foo", "salt")


# 'get' function tests: 2


def test_get():
    """
    Test if it gets a value from the minion datastore
    """
    with patch("salt.modules.data.load", MagicMock(return_value={"salt": "SALT"})):
        assert data.get("salt") == "SALT"


def test_get_vals():
    """
    Test if it gets values from the minion datastore
    """
    with patch(
        "salt.modules.data.load",
        MagicMock(return_value={"salt": "SALT", "salt1": "SALT1"}),
    ):
        assert data.get(["salt", "salt1"]) == ["SALT", "SALT1"]


# 'cas' function tests: 1


def test_cas_not_load():
    """
    Test if it check and set a value in the minion datastore
    """
    with patch(
        "salt.modules.data.load",
        MagicMock(return_value={"salt": "SALT", "salt1": "SALT1"}),
    ):
        assert not data.cas("salt3", "SALT", "SALTSTACK")


def test_cas_not_equal():
    """
    Test if it check and set a value in the minion datastore
    """
    with patch(
        "salt.modules.data.load",
        MagicMock(return_value={"salt": "SALT", "salt1": "SALT1"}),
    ):
        assert not data.cas("salt", "SALT", "SALTSTACK")


def test_cas():
    """
    Test if it check and set a value in the minion datastore
    """
    with patch(
        "salt.modules.data.load",
        MagicMock(return_value={"salt": "SALT", "salt1": "SALT1"}),
    ), patch("salt.modules.data.dump", MagicMock(return_value=True)):
        assert data.cas("salt", "SALTSTACK", "SALT")



import copy
from collections import OrderedDict

import salt.utils.dictupdate as dictupdate
from salt.exceptions import SaltInvocationError
from tests.support.unit import TestCase


class UtilDictupdateTestCase(TestCase):

    dict1 = {"A": "B", "C": {"D": "E", "F": {"G": "H", "I": "J"}}}

    def test_update(self):

        # level 1 value changes
        mdict = copy.deepcopy(self.dict1)
        mdict["A"] = "Z"
        res = dictupdate.update(copy.deepcopy(self.dict1), {"A": "Z"})
        self.assertEqual(res, mdict)

        # level 1 value changes (list replacement)
        mdict = copy.deepcopy(self.dict1)
        mdict["A"] = [1, 2]
        res = dictupdate.update(copy.deepcopy(mdict), {"A": [2, 3]}, merge_lists=False)
        mdict["A"] = [2, 3]
        self.assertEqual(res, mdict)

        # level 1 value changes (list merge)
        mdict = copy.deepcopy(self.dict1)
        mdict["A"] = [1, 2]
        res = dictupdate.update(copy.deepcopy(mdict), {"A": [3, 4]}, merge_lists=True)
        mdict["A"] = [1, 2, 3, 4]
        self.assertEqual(res, mdict)

        # level 1 value changes (list merge, remove duplicates, preserve order)
        mdict = copy.deepcopy(self.dict1)
        mdict["A"] = [1, 2]
        res = dictupdate.update(
            copy.deepcopy(mdict), {"A": [4, 3, 2, 1]}, merge_lists=True
        )
        mdict["A"] = [1, 2, 4, 3]
        self.assertEqual(res, mdict)

        # level 2 value changes
        mdict = copy.deepcopy(self.dict1)
        mdict["C"]["D"] = "Z"
        res = dictupdate.update(copy.deepcopy(self.dict1), {"C": {"D": "Z"}})
        self.assertEqual(res, mdict)

        # level 2 value changes (list replacement)
        mdict = copy.deepcopy(self.dict1)
        mdict["C"]["D"] = ["a", "b"]
        res = dictupdate.update(
            copy.deepcopy(mdict), {"C": {"D": ["c", "d"]}}, merge_lists=False
        )
        mdict["C"]["D"] = ["c", "d"]
        self.assertEqual(res, mdict)

        # level 2 value changes (list merge)
        mdict = copy.deepcopy(self.dict1)
        mdict["C"]["D"] = ["a", "b"]
        res = dictupdate.update(
            copy.deepcopy(mdict), {"C": {"D": ["c", "d"]}}, merge_lists=True
        )
        mdict["C"]["D"] = ["a", "b", "c", "d"]
        self.assertEqual(res, mdict)

        # level 2 value changes (list merge, remove duplicates, preserve order)
        mdict = copy.deepcopy(self.dict1)
        mdict["C"]["D"] = ["a", "b"]
        res = dictupdate.update(
            copy.deepcopy(mdict), {"C": {"D": ["d", "c", "b", "a"]}}, merge_lists=True
        )
        mdict["C"]["D"] = ["a", "b", "d", "c"]
        self.assertEqual(res, mdict)

        # level 3 value changes
        mdict = copy.deepcopy(self.dict1)
        mdict["C"]["F"]["G"] = "Z"
        res = dictupdate.update(copy.deepcopy(self.dict1), {"C": {"F": {"G": "Z"}}})
        self.assertEqual(res, mdict)

        # level 3 value changes (list replacement)
        mdict = copy.deepcopy(self.dict1)
        mdict["C"]["F"]["G"] = ["a", "b"]
        res = dictupdate.update(
            copy.deepcopy(mdict), {"C": {"F": {"G": ["c", "d"]}}}, merge_lists=False
        )
        mdict["C"]["F"]["G"] = ["c", "d"]
        self.assertEqual(res, mdict)

        # level 3 value changes (list merge)
        mdict = copy.deepcopy(self.dict1)
        mdict["C"]["F"]["G"] = ["a", "b"]
        res = dictupdate.update(
            copy.deepcopy(mdict), {"C": {"F": {"G": ["c", "d"]}}}, merge_lists=True
        )
        mdict["C"]["F"]["G"] = ["a", "b", "c", "d"]
        self.assertEqual(res, mdict)

        # level 3 value changes (list merge, remove duplicates, preserve order)
        mdict = copy.deepcopy(self.dict1)
        mdict["C"]["F"]["G"] = ["a", "b"]
        res = dictupdate.update(
            copy.deepcopy(mdict),
            {"C": {"F": {"G": ["d", "c", "b", "a"]}}},
            merge_lists=True,
        )
        mdict["C"]["F"]["G"] = ["a", "b", "d", "c"]
        self.assertEqual(res, mdict)

        # replace a sub-dictionary
        mdict = copy.deepcopy(self.dict1)
        mdict["C"] = "Z"
        res = dictupdate.update(copy.deepcopy(self.dict1), {"C": "Z"})
        self.assertEqual(res, mdict)

        # add a new scalar value
        mdict = copy.deepcopy(self.dict1)
        mdict["Z"] = "Y"
        res = dictupdate.update(copy.deepcopy(self.dict1), {"Z": "Y"})
        self.assertEqual(res, mdict)

        # add a dictionary
        mdict = copy.deepcopy(self.dict1)
        mdict["Z"] = {"Y": "X"}
        res = dictupdate.update(copy.deepcopy(self.dict1), {"Z": {"Y": "X"}})
        self.assertEqual(res, mdict)

        # add a nested dictionary
        mdict = copy.deepcopy(self.dict1)
        mdict["Z"] = {"Y": {"X": "W"}}
        res = dictupdate.update(copy.deepcopy(self.dict1), {"Z": {"Y": {"X": "W"}}})
        self.assertEqual(res, mdict)


class UtilDictMergeTestCase(TestCase):

    dict1 = {"A": "B", "C": {"D": "E", "F": {"G": "H", "I": "J"}}}

    def test_merge_overwrite_traditional(self):
        """
        Test traditional overwrite, wherein a key in the second dict overwrites a key in the first
        """
        mdict = copy.deepcopy(self.dict1)
        mdict["A"] = "b"
        ret = dictupdate.merge_overwrite(copy.deepcopy(self.dict1), {"A": "b"})
        self.assertEqual(mdict, ret)

    def test_merge_overwrite_missing_source_key(self):
        """
        Test case wherein the overwrite strategy is used but a key in the second dict is
        not present in the first
        """
        mdict = copy.deepcopy(self.dict1)
        mdict["D"] = "new"
        ret = dictupdate.merge_overwrite(copy.deepcopy(self.dict1), {"D": "new"})
        self.assertEqual(mdict, ret)

    def test_merge_aggregate_traditional(self):
        """
        Test traditional aggregation, where a val from dict2 overwrites one
        present in dict1
        """
        mdict = copy.deepcopy(self.dict1)
        mdict["A"] = "b"
        ret = dictupdate.merge_overwrite(copy.deepcopy(self.dict1), {"A": "b"})
        self.assertEqual(mdict, ret)

    def test_merge_list_traditional(self):
        """
        Test traditional list merge, where a key present in dict2 will be converted
        to a list
        """
        mdict = copy.deepcopy(self.dict1)
        mdict["A"] = ["B", "b"]
        ret = dictupdate.merge_list(copy.deepcopy(self.dict1), {"A": "b"})
        self.assertEqual(mdict, ret)

    def test_merge_list_append(self):
        """
        This codifies the intended behaviour that items merged into a dict val that is already
        a list that those items will *appended* to the list, and not magically merged in
        """
        mdict = copy.deepcopy(self.dict1)
        mdict["A"] = ["B", "b", "c"]

        # Prepare a modified copy of dict1 that has a list as a val for the key of 'A'
        mdict1 = copy.deepcopy(self.dict1)
        mdict1["A"] = ["B"]
        ret = dictupdate.merge_list(mdict1, {"A": ["b", "c"]})
        self.assertEqual(
            {"A": [["B"], ["b", "c"]], "C": {"D": "E", "F": {"I": "J", "G": "H"}}}, ret
        )


class UtilDeepDictUpdateTestCase(TestCase):

    dict1 = {"A": "B", "C": {"D": "E", "F": {"G": "H", "I": "J"}}}

    def test_deep_set_overwrite(self):
        """
        Test overwriting an existing value.
        """
        mdict = copy.deepcopy(self.dict1)
        res = dictupdate.set_dict_key_value(mdict, "C:F", "foo")
        self.assertEqual({"A": "B", "C": {"D": "E", "F": "foo"}}, res)
        # Verify modify-in-place
        self.assertEqual({"A": "B", "C": {"D": "E", "F": "foo"}}, mdict)

        # Test using alternative delimiter
        res = dictupdate.set_dict_key_value(
            mdict, "C/F", {"G": "H", "I": "J"}, delimiter="/"
        )
        self.assertEqual(self.dict1, res)

        # Test without using a delimiter in the keys
        res = dictupdate.set_dict_key_value(mdict, "C", None)
        self.assertEqual({"A": "B", "C": None}, res)

    def test_deep_set_create(self):
        """
        Test creating new nested keys.
        """
        mdict = copy.deepcopy(self.dict1)
        res = dictupdate.set_dict_key_value(mdict, "K:L:M", "Q")
        self.assertEqual(
            {
                "A": "B",
                "C": {"D": "E", "F": {"G": "H", "I": "J"}},
                "K": {"L": {"M": "Q"}},
            },
            res,
        )

    def test_deep_set_ordered_dicts(self):
        """
        Test creating new nested ordereddicts.
        """
        res = dictupdate.set_dict_key_value({}, "A:B", "foo", ordered_dict=True)
        self.assertEqual({"A": OrderedDict([("B", "foo")])}, res)

    def test_deep_append(self):
        """
        Test appending to a list.
        """
        sdict = {"bar": {"baz": [1, 2]}}
        res = dictupdate.append_dict_key_value(sdict, "bar:baz", 42)
        self.assertEqual({"bar": {"baz": [1, 2, 42]}}, res)
        # Append with alternate delimiter
        res = dictupdate.append_dict_key_value(sdict, "bar~baz", 43, delimiter="~")
        self.assertEqual({"bar": {"baz": [1, 2, 42, 43]}}, res)
        # Append to a not-yet existing list
        res = dictupdate.append_dict_key_value({}, "foo:bar:baz", 42)
        self.assertEqual({"foo": {"bar": {"baz": [42]}}}, res)

    def test_deep_extend(self):
        """
        Test extending a list.
        Note that the provided value (to extend with) will be coerced to a list
        if this is not already a list. This can cause unexpected behaviour.
        """
        sdict = {"bar": {"baz": [1, 2]}}
        res = dictupdate.extend_dict_key_value(sdict, "bar:baz", [42, 42])
        self.assertEqual({"bar": {"baz": [1, 2, 42, 42]}}, res)

        # Extend a not-yet existing list
        res = dictupdate.extend_dict_key_value({}, "bar:baz:qux", [42])
        self.assertEqual({"bar": {"baz": {"qux": [42]}}}, res)

        # Extend with a dict (remember, foo has been updated in the first test)
        res = dictupdate.extend_dict_key_value(sdict, "bar:baz", {"qux": "quux"})
        self.assertEqual({"bar": {"baz": [1, 2, 42, 42, "qux"]}}, res)

    def test_deep_extend_illegal_addition(self):
        """
        Test errorhandling extending lists with illegal types.
        """
        # Extend with an illegal type
        for extend_with in [42, None]:
            with self.assertRaisesRegex(
                SaltInvocationError,
                r"Cannot extend {} with a {}." "".format(type([]), type(extend_with)),
            ):
                dictupdate.extend_dict_key_value({}, "foo", extend_with)

    def test_deep_extend_illegal_source(self):
        """
        Test errorhandling extending things that are not a list.
        """
        # Extend an illegal type
        for extend_this in [{}, 42, "bar"]:
            with self.assertRaisesRegex(
                SaltInvocationError,
                r"The last key contains a {}, which cannot extend."
                "".format(type(extend_this)),
            ):
                dictupdate.extend_dict_key_value({"foo": extend_this}, "foo", [42])

    def test_deep_update(self):
        """
        Test updating a (sub)dict.
        """
        mdict = copy.deepcopy(self.dict1)
        res = dictupdate.update_dict_key_value(
            mdict, "C:F", {"foo": "bar", "qux": "quux"}
        )
        self.assertEqual(
            {
                "A": "B",
                "C": {"D": "E", "F": {"G": "H", "I": "J", "foo": "bar", "qux": "quux"}},
            },
            res,
        )

        # Test updating a non-existing subkey
        res = dictupdate.update_dict_key_value({}, "foo:bar:baz", {"qux": "quux"})
        self.assertEqual({"foo": {"bar": {"baz": {"qux": "quux"}}}}, res)
        # Test updating a non-existing subkey, with a different delimiter
        res = dictupdate.update_dict_key_value(
            {}, "foo bar baz", {"qux": "quux"}, delimiter=" "
        )
        self.assertEqual({"foo": {"bar": {"baz": {"qux": "quux"}}}}, res)

    def test_deep_update_illegal_update(self):
        """
        Test errorhandling updating a (sub)dict with illegal types.
        """
        # Update with an illegal type
        for update_with in [42, None, [42], "bar"]:
            with self.assertRaisesRegex(
                SaltInvocationError,
                r"Cannot update {} with a {}." "".format(type({}), type(update_with)),
            ):
                dictupdate.update_dict_key_value({}, "foo", update_with)
        # Again, but now using OrderedDicts
        for update_with in [42, None, [42], "bar"]:
            with self.assertRaisesRegex(
                SaltInvocationError,
                r"Cannot update {} with a {}."
                "".format(type(OrderedDict()), type(update_with)),
            ):
                dictupdate.update_dict_key_value(
                    {}, "foo", update_with, ordered_dict=True
                )

    def test_update_with_merge_lists_and_mapping(self):
        """
        Test that updating a list with a mapping raises a TypeError
        when merge_lists=True.
        """
        mdict = copy.deepcopy(self.dict1)
        mdict["A"] = [1, 2]
        with self.assertRaises(TypeError):
            dictupdate.update(
                copy.deepcopy(mdict),
                {"A": {"key": "value"}},
                merge_lists=True,
                strict=True,
            )



"""
Verify AsyncAuth.authenticate publishes salt/auth/creds synchronously when
auth_events=True. This guards against the macOS regression where the event was
fired via fire_event_async on the calling subprocess io_loop and got lost
before the parent minion's handle_event consumer could update creds_map.
"""

import os

import pytest

import salt.crypt as crypt
from tests.support.mock import MagicMock, patch


@pytest.fixture
def minion_root(tmp_path):
    root = tmp_path / "root"
    root.mkdir()
    (root / "etc").mkdir()
    (root / "etc" / "salt").mkdir()
    (root / "etc" / "salt" / "pki").mkdir()
    yield root


async def test_authenticate_fires_creds_event_synchronously(minion_root, io_loop):
    """
    With auth_events=True, the salt/auth/creds event MUST be published via the
    synchronous event.fire_event() (not fire_event_async with io_loop=...) so
    that the publish completes before the `with` block tears down the IPC
    session, regardless of which io_loop the caller is on.
    """
    pki_dir = minion_root / "etc" / "salt" / "pki"
    os.makedirs(str(pki_dir), exist_ok=True)
    opts = {
        "id": "minion",
        "__role": "minion",
        "auth_events": True,
        "pki_dir": str(pki_dir),
        "master_uri": "tcp://127.0.0.1:4505",
        "keysize": 4096,
        "acceptance_wait_time": 60,
        "keys.cache_driver": "localfs_key",
        "acceptance_wait_time_max": 60,
    }
    priv, pub = crypt.gen_keys(opts["keysize"])
    keypath = pki_dir / "minion"
    keypath.with_suffix(".pem").write_text(priv)
    keypath.with_suffix(".pub").write_text(pub)

    aes = crypt.Crypticle.generate_key_string()
    session = crypt.Crypticle.generate_key_string()

    auth = crypt.AsyncAuth(opts, io_loop)

    async def mock_sign_in(*args, **kwargs):
        return mock_sign_in.response

    mock_sign_in.response = {"enc": "pub", "aes": aes, "session": session}
    auth.sign_in = mock_sign_in

    # Capture how the event bus is used by AsyncAuth.authenticate()
    fake_event = MagicMock()
    cm = MagicMock()
    cm.__enter__ = MagicMock(return_value=fake_event)
    cm.__exit__ = MagicMock(return_value=False)
    get_event_mock = MagicMock(return_value=cm)

    with patch("salt.utils.event.get_event", get_event_mock):
        await auth.authenticate()

    # The IPC session must be opened with the role's bus and NO io_loop, so
    # the publish runs synchronously on the active loop and completes before
    # the context manager exits (the bug was passing io_loop=self.io_loop
    # which scheduled the publish on a coro that may never get awaited from
    # a short-lived subprocess).
    get_event_mock.assert_called_once()
    call_kwargs = get_event_mock.call_args.kwargs
    assert "io_loop" not in call_kwargs, (
        "get_event must NOT be passed io_loop; the event must fire on the "
        "role's own bus, synchronously"
    )
    assert call_kwargs.get("listen") is False

    # Must be the SYNC fire_event(...), NOT fire_event_async(...).
    fake_event.fire_event.assert_called_once()
    assert not fake_event.fire_event_async.called, (
        "auth/creds must be fired synchronously via event.fire_event, not "
        "fire_event_async"
    )

    # Sanity-check the event payload/tag.
    args, _ = fake_event.fire_event.call_args
    payload, tag = args
    assert tag == "salt/auth/creds"
    assert payload["creds"]["aes"] == aes
    assert payload["creds"]["session"] == session



import logging
import os
import shutil

import pytest

import salt.config
import salt.loader
import salt.modules.cmdmod as cmdmod
import salt.modules.config as configmod
import salt.modules.file as filemod
import salt.utils.data
import salt.utils.files
import salt.utils.platform
import salt.utils.stringutils
from tests.support.mock import MagicMock, call, patch

log = logging.getLogger(__name__)


@pytest.fixture
def configure_loader_modules():
    return {
        filemod: {
            "__salt__": {
                "config.manage_mode": configmod.manage_mode,
                "cmd.run": cmdmod.run,
                "cmd.run_all": cmdmod.run_all,
            },
            "__opts__": {
                "test": False,
                "file_roots": {"base": "tmp"},
                "pillar_roots": {"base": "tmp"},
                "cachedir": "tmp",
                "grains": {},
            },
            "__grains__": {"kernel": "Linux"},
        }
    }


@pytest.fixture
def tmp_sub_dir(tmp_path):
    directory = tmp_path / "file-basics-test-dir"
    directory.mkdir()

    yield directory

    shutil.rmtree(str(directory))


@pytest.fixture
def tfile(tmp_sub_dir):
    filename = str(tmp_sub_dir / "file-basics-test-file")

    with salt.utils.files.fopen(filename, "w+") as fp:
        fp.write("Hi hello! I am a file.")

    yield filename

    os.remove(filename)


@pytest.fixture
def myfile(tmp_sub_dir):
    filename = str(tmp_sub_dir / "myfile")

    with salt.utils.files.fopen(filename, "w+") as fp:
        fp.write(salt.utils.stringutils.to_str("Hello\n"))

    yield filename

    os.remove(filename)


@pytest.fixture
def a_link(tmp_sub_dir):
    path = tmp_sub_dir / "a_link"
    linkname = str(path)

    yield linkname

    if path.exists():
        os.remove(linkname)


@pytest.fixture
def a_hardlink(tmp_sub_dir):
    path = tmp_sub_dir / "a_hardlink"
    linkname = str(path)

    yield linkname

    if path.exists():
        os.remove(linkname)


@pytest.mark.skip_on_windows(reason="os.symlink is not available on Windows")
def test_symlink_already_in_desired_state(tfile, a_link):
    os.symlink(tfile, a_link)
    result = filemod.symlink(tfile, a_link)
    assert result


@pytest.mark.skip_on_windows(reason="os.link is not available on Windows")
def test_hardlink_sanity(tfile, a_hardlink):
    target = a_hardlink
    result = filemod.link(tfile, target)
    assert result


@pytest.mark.skip_on_windows(reason="os.link is not available on Windows")
def test_hardlink_numlinks(tfile, a_hardlink):
    target = a_hardlink
    result = filemod.link(tfile, target)
    name_i = os.stat(tfile).st_nlink
    assert name_i > 1


@pytest.mark.skip_on_windows(reason="os.link is not available on Windows")
def test_hardlink_working(tfile, a_hardlink):
    target = a_hardlink
    result = filemod.link(tfile, target)
    name_i = os.stat(tfile).st_ino
    target_i = os.stat(target).st_ino
    assert name_i == target_i


def test_source_list_for_list_returns_file_from_dict_via_http():
    with patch("salt.modules.file.os.remove") as remove:
        remove.return_value = None
        with patch.dict(
            filemod.__salt__,
            {
                "cp.list_master": MagicMock(return_value=[]),
                "cp.list_master_dirs": MagicMock(return_value=[]),
                "cp.cache_file": MagicMock(return_value="/tmp/http.conf"),
            },
        ):
            with patch("salt.utils.http.query") as http_query:
                http_query.return_value = {}
                ret = filemod.source_list(
                    [{"http://t.est.com/http/httpd.conf": "filehash"}], "", "base"
                )
                assert list(ret) == ["http://t.est.com/http/httpd.conf", "filehash"]


def test_source_list_use_requests():
    with patch("salt.modules.file.os.remove") as remove:
        remove.return_value = None
        with patch.dict(
            filemod.__salt__,
            {
                "cp.list_master": MagicMock(return_value=[]),
                "cp.list_master_dirs": MagicMock(return_value=[]),
                "cp.cache_file": MagicMock(return_value="/tmp/http.conf"),
            },
        ):
            expected_call = call(
                "http://t.est.com/http/file1",
                decode_body=False,
                method="HEAD",
            )
            with patch(
                "salt.utils.http.query", MagicMock(return_value={})
            ) as http_query:
                ret = filemod.source_list(
                    [{"http://t.est.com/http/file1": "filehash"}], "", "base"
                )
                assert list(ret) == ["http://t.est.com/http/file1", "filehash"]
                assert expected_call in http_query.mock_calls


def test_source_list_for_list_returns_existing_file():
    with patch.dict(
        filemod.__salt__,
        {
            "cp.list_master": MagicMock(return_value=["http/httpd.conf.fallback"]),
            "cp.list_master_dirs": MagicMock(return_value=[]),
        },
    ):
        ret = filemod.source_list(
            ["salt://http/httpd.conf", "salt://http/httpd.conf.fallback"],
            "filehash",
            "base",
        )
        assert list(ret) == ["salt://http/httpd.conf.fallback", "filehash"]


def test_source_list_for_list_returns_file_from_other_env():
    def list_master(env):
        dct = {"base": [], "dev": ["http/httpd.conf"]}
        return dct[env]

    with patch.dict(
        filemod.__salt__,
        {
            "cp.list_master": MagicMock(side_effect=list_master),
            "cp.list_master_dirs": MagicMock(return_value=[]),
        },
    ):
        ret = filemod.source_list(
            [
                "salt://http/httpd.conf?saltenv=dev",
                "salt://http/httpd.conf.fallback",
            ],
            "filehash",
            "base",
        )
        assert list(ret) == ["salt://http/httpd.conf?saltenv=dev", "filehash"]


def test_source_list_for_list_returns_file_from_dict():
    with patch.dict(
        filemod.__salt__,
        {
            "cp.list_master": MagicMock(return_value=["http/httpd.conf"]),
            "cp.list_master_dirs": MagicMock(return_value=[]),
        },
    ):
        ret = filemod.source_list([{"salt://http/httpd.conf": ""}], "filehash", "base")
        assert list(ret) == ["salt://http/httpd.conf", "filehash"]


def test_source_list_for_list_returns_existing_local_file_slash(myfile):
    with patch.dict(
        filemod.__salt__,
        {
            "cp.list_master": MagicMock(return_value=[]),
            "cp.list_master_dirs": MagicMock(return_value=[]),
        },
    ):
        ret = filemod.source_list([myfile + "-foo", myfile], "filehash", "base")
        assert list(ret) == [myfile, "filehash"]


def test_source_list_for_list_returns_existing_local_file_proto(myfile):
    with patch.dict(
        filemod.__salt__,
        {
            "cp.list_master": MagicMock(return_value=[]),
            "cp.list_master_dirs": MagicMock(return_value=[]),
        },
    ):
        ret = filemod.source_list(
            ["file://" + myfile + "-foo", "file://" + myfile],
            "filehash",
            "base",
        )
        assert list(ret) == ["file://" + myfile, "filehash"]


def test_source_list_for_list_returns_local_file_slash_from_dict(myfile):
    with patch.dict(
        filemod.__salt__,
        {
            "cp.list_master": MagicMock(return_value=[]),
            "cp.list_master_dirs": MagicMock(return_value=[]),
        },
    ):
        ret = filemod.source_list([{myfile: ""}], "filehash", "base")
        assert list(ret) == [myfile, "filehash"]


def test_source_list_for_list_returns_local_file_proto_from_dict(myfile):
    with patch.dict(
        filemod.__salt__,
        {
            "cp.list_master": MagicMock(return_value=[]),
            "cp.list_master_dirs": MagicMock(return_value=[]),
        },
    ):
        ret = filemod.source_list([{"file://" + myfile: ""}], "filehash", "base")
        assert list(ret) == ["file://" + myfile, "filehash"]


def test_symlink_lexists_called_follow_symlinks_false():
    tfile = "/tmp/file-basics-test-file"
    a_link = "/tmp/a_link"

    exists = MagicMock(return_value=False)
    lexists = MagicMock(return_value=False)

    with patch("os.path.exists", exists), patch("os.path.lexists", lexists), patch(
        "os.symlink", MagicMock(return_value=True)
    ):
        filemod.symlink(tfile, a_link)
        lexists.assert_not_called()
        exists.assert_called()

        lexists.reset_mock()
        exists.reset_mock()

        filemod.symlink(tfile, a_link, follow_symlinks=False)
        lexists.assert_called()
        exists.assert_not_called()


def test_symlink_lexists_called_follow_symlinks_true():
    tfile = "/tmp/file-basics-test-file"
    a_link = "/tmp/a_link"

    exists = MagicMock(return_value=False)
    lexists = MagicMock(return_value=False)

    with patch("os.path.exists", exists), patch("os.path.lexists", lexists), patch(
        "os.symlink", MagicMock(return_value=True)
    ):
        filemod.symlink(tfile, a_link, follow_symlinks=True)
        lexists.assert_not_called()
        exists.assert_called()



import pytest

from salt.cloud import Cloud
from salt.config import get_cloud_config_value
from salt.exceptions import SaltCloudSystemExit
from tests.support.mock import MagicMock, patch

pytestmark = [
    pytest.mark.timeout_unless_on_windows(120),
]


@pytest.fixture
def master_config(master_opts):
    master_opts["parallel"] = False
    master_opts["providers"] = {
        "test": {},
    }
    return master_opts


@pytest.fixture
def vm_config():
    return {
        "driver": "test",
        "name": "test",
        "provider": "test:test",
    }


@pytest.mark.parametrize(
    "sync, expected_func",
    (
        ("all", "saltutil.sync_all"),
        ("beacons", "saltutil.sync_beacons"),
        ("clouds", "saltutil.sync_clouds"),
        ("engines", "saltutil.sync_engines"),
        ("executors", "saltutil.sync_executors"),
        ("grains", "saltutil.sync_grains"),
        ("log", "saltutil.sync_log"),
        ("matchers", "saltutil.sync_matchers"),
        ("modules", "saltutil.sync_modules"),
        ("output", "saltutil.sync_output"),
        ("pillar", "saltutil.sync_pillar"),
        ("proxymodules", "saltutil.sync_proxymodules"),
        ("renderers", "saltutil.sync_renderers"),
        ("returners", "saltutil.sync_returners"),
        ("sdb", "saltutil.sync_sdb"),
        ("serializers", "saltutil.sync_serializers"),
        ("states", "saltutil.sync_states"),
        ("thorium", "saltutil.sync_thorium"),
        ("utils", "saltutil.sync_utils"),
        (
            "lol this is a bad sync option",
            "saltutil.sync_all",
        ),  # With a bad option it should default to all
    ),
)
def test_cloud_create_attempt_sync_after_install(
    master_config, vm_config, sync, expected_func
):
    master_config["sync_after_install"] = sync
    cloud = Cloud(master_config)
    cloud.clouds["test.create"] = lambda x: True

    fake_context_manager = MagicMock()
    fake_client = MagicMock(return_value=MagicMock(return_value=True))
    fake_context_manager.__enter__.return_value = fake_client
    with patch(
        "salt.client.get_local_client",
        autospec=True,
        return_value=fake_context_manager,
    ):
        ret = cloud.create(vm_config, sync_sleep=0)
    assert ret
    fake_client.cmd.assert_called_with("test", expected_func, timeout=5)


@pytest.mark.slow_test
def test_vm_config_merger():
    """
    Validate the vm's config is generated correctly.

    https://github.com/saltstack/salt/issues/49226
    https://github.com/saltstack/salt/issues/64610
    """
    main = {
        "minion": {"master": "172.31.39.213"},
        "log_file": "var/log/salt/cloud.log",
        "pool_size": 10,
    }
    provider = {
        "private_key": "dwoz.pem",
        "grains": {"foo1": "bar", "foo2": "bang"},
        "availability_zone": "us-west-2b",
        "driver": "ec2",
        "ssh_interface": "private_ips",
        "ssh_username": "admin",
        "location": "us-west-2",
    }
    profile = {
        "profile": "default",
        "grains": {"meh2": "bar", "meh1": "foo"},
        "provider": "ec2-default:ec2",
        "ssh_username": "admin",
        "image": "ami-0a1fbca0e5b419fd1",
        "size": "t2.micro",
    }
    vm_overrides = {
        "test_vm": {"grains": {"meh2": "newbar", "meh3": "foo"}},
        "other_vm": {"grains": {"meh1": "notused"}},
    }
    expected = {
        "minion": {"master": "172.31.39.213"},
        "log_file": "var/log/salt/cloud.log",
        "pool_size": 10,
        "private_key": "dwoz.pem",
        "grains": {
            "foo1": "bar",
            "foo2": "bang",
            "meh3": "foo",
            "meh2": "newbar",
            "meh1": "foo",
        },
        "availability_zone": "us-west-2b",
        "driver": "ec2",
        "ssh_interface": "private_ips",
        "ssh_username": "admin",
        "location": "us-west-2",
        "profile": "default",
        "provider": "ec2-default:ec2",
        "image": "ami-0a1fbca0e5b419fd1",
        "size": "t2.micro",
        "name": "test_vm",
    }
    vm = Cloud.vm_config("test_vm", main, provider, profile, vm_overrides)
    assert expected == vm


@pytest.mark.slow_test
def test_vm_config_merger_nooverridevalue():
    """
    Validate the vm's config is generated correctly, even if no
    applicable values are in the vm_override structure

    https://github.com/saltstack/salt/issues/64610
    """
    main = {
        "minion": {"master": "172.31.39.213"},
        "log_file": "var/log/salt/cloud.log",
        "pool_size": 10,
    }
    provider = {
        "private_key": "dwoz.pem",
        "grains": {"foo1": "bar", "foo2": "bang"},
        "availability_zone": "us-west-2b",
        "driver": "ec2",
        "ssh_interface": "private_ips",
        "ssh_username": "admin",
        "location": "us-west-2",
    }
    profile = {
        "profile": "default",
        "grains": {"meh2": "bar", "meh1": "foo"},
        "provider": "ec2-default:ec2",
        "ssh_username": "admin",
        "image": "ami-0a1fbca0e5b419fd1",
        "size": "t2.micro",
    }
    vm_overrides = {
        "test_vm": {"grains": {"meh2": "newbar", "meh3": "foo"}},
        "other_vm": {"grains": {"meh1": "notused"}},
    }
    expected = {
        "minion": {"master": "172.31.39.213"},
        "log_file": "var/log/salt/cloud.log",
        "pool_size": 10,
        "private_key": "dwoz.pem",
        "grains": {
            "foo1": "bar",
            "foo2": "bang",
            "meh2": "bar",
            "meh1": "foo",
        },
        "availability_zone": "us-west-2b",
        "driver": "ec2",
        "ssh_interface": "private_ips",
        "ssh_username": "admin",
        "location": "us-west-2",
        "profile": "default",
        "provider": "ec2-default:ec2",
        "image": "ami-0a1fbca0e5b419fd1",
        "size": "t2.micro",
        "name": "test_vm2",
    }
    vm = Cloud.vm_config("test_vm2", main, provider, profile, vm_overrides)
    assert expected == vm


@pytest.mark.skip_on_fips_enabled_platform
def test_cloud_run_profile_create_returns_boolean(master_config):
    master_config["profiles"] = {"test_profile": {"provider": "test_provider:saltify"}}
    master_config["providers"] = {
        "test_provider": {
            "saltify": {"profiles": {"provider": "test_provider:saltify"}}
        }
    }
    master_config["show_deploy_args"] = False

    cloud = Cloud(master_config)
    with patch.object(cloud, "create", return_value=True):
        ret = cloud.run_profile("test_profile", ["test_vm"])
        assert ret == {"test_vm": True}

    cloud = Cloud(master_config)
    with patch.object(cloud, "create", return_value=False):
        with pytest.raises(SaltCloudSystemExit):
            ret = cloud.run_profile("test_profile", ["test_vm"])
            assert ret == {"test_vm": False}


@pytest.mark.parametrize(
    "value",
    [
        [{"key1": "value1"}, {"key1": "value1", "key2": "value2"}],
        ["a", "b"],
        [1, 2, 4],
        {"key1": "value1", "key2": 123},
        "some text",
        1234,
    ],
)
def test_get_cloud_config_value(value):
    value_name = "test_value_name"
    opts = {
        "providers": {
            "my-cool-cloud-provider": {
                "cool-cloud": {
                    "driver": "cool-cloud",
                    "profiles": {
                        "my-cool-cloud-profile": {
                            "provider": "my-cool-cloud-provider:cool-cloud",
                            value_name: value,
                            "profile": "my-cool-cloud-profile",
                        }
                    },
                }
            }
        },
        "profiles": {
            "my-cool-cloud-profile": {
                "provider": "my-cool-cloud-provider:cool-cloud",
                value_name: value,
                "profile": "my-cool-cloud-profile",
            }
        },
        "profile": "my-cool-cloud-profile",
    }
    vm_ = {
        value_name: value,
        "profile": "my-cool-cloud-profile",
        "driver": "cool-cloud",
    }

    result = get_cloud_config_value(value_name, vm_, opts)
    assert result == value



import pytest

import salt.exceptions
import salt.utils.virt


def test_virt_key(tmp_path):
    opts = {"pki_dir": f"{tmp_path / 'pki'}"}
    salt.utils.virt.VirtKey("asdf", "minion", opts)


def test_virt_key_bad_hyper(tmp_path):
    opts = {"pki_dir": f"{tmp_path / 'pki'}"}
    with pytest.raises(salt.exceptions.SaltValidationError):
        salt.utils.virt.VirtKey("asdf/../../../sdf", "minion", opts)


def test_virt_key_bad_id_(tmp_path):
    opts = {"pki_dir": f"{tmp_path / 'pki'}"}
    with pytest.raises(salt.exceptions.SaltValidationError):
        salt.utils.virt.VirtKey("hyper", "minion/../../", opts)



"""
    :codeauthor: Ted Strzalkowski (tedski@gmail.com)


    tests.unit.modules.mdadm_test
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
"""

import re

import pytest

import salt.modules.mdadm_raid as mdadm
from tests.support.mock import MagicMock, patch


@pytest.fixture
def configure_loader_modules():
    return {mdadm: {}}


def test_create():
    mock = MagicMock(return_value="salt")
    with patch.dict(mdadm.__salt__, {"cmd.run": mock}), patch(
        "salt.utils.path.which", lambda exe: exe
    ):
        ret = mdadm.create(
            "/dev/md0",
            5,
            devices=["/dev/sdb1", "/dev/sdc1", "/dev/sdd1"],
            test_mode=False,
            force=True,
            chunk=256,
        )
        assert "salt" == ret
        mock.assert_called_with(
            [
                "mdadm",
                "-C",
                "/dev/md0",
                "-R",
                "-v",
                "-l",
                "5",
                "--force",
                "--chunk",
                "256",
                "-e",
                "default",
                "-n",
                "3",
                "/dev/sdb1",
                "/dev/sdc1",
                "/dev/sdd1",
            ],
            python_shell=False,
        )


def test_create_metadata():
    mock = MagicMock(return_value="salt")
    with patch.dict(mdadm.__salt__, {"cmd.run": mock}), patch(
        "salt.utils.path.which", lambda exe: exe
    ):
        ret = mdadm.create(
            "/dev/md0",
            5,
            devices=["/dev/sdb1", "/dev/sdc1", "/dev/sdd1"],
            metadata=0.9,
            test_mode=False,
            force=True,
            chunk=256,
        )
        assert "salt" == ret
        mock.assert_called_with(
            [
                "mdadm",
                "-C",
                "/dev/md0",
                "-R",
                "-v",
                "-l",
                "5",
                "--force",
                "--chunk",
                "256",
                "-e",
                "0.9",
                "-n",
                "3",
                "/dev/sdb1",
                "/dev/sdc1",
                "/dev/sdd1",
            ],
            python_shell=False,
        )


def test_create_test_mode():
    mock = MagicMock()
    with patch.dict(mdadm.__salt__, {"cmd.run": mock}):
        ret = mdadm.create(
            "/dev/md0",
            5,
            devices=["/dev/sdb1", "/dev/sdc1", "/dev/sdd1"],
            force=True,
            chunk=256,
            test_mode=True,
        )
        assert sorted(
            "mdadm -C /dev/md0 -R -v --chunk 256 "
            "--force -l 5 -e default -n 3 "
            "/dev/sdb1 /dev/sdc1 /dev/sdd1".split()
        ) == sorted(ret.split())
        assert not mock.called, "test mode failed, cmd.run called"


def test_examine():
    """
    Test for mdadm_raid.examine
    """
    mock = MagicMock(
        return_value=(
            "ARRAY /dev/md/pool metadata=1.2"
            " UUID=567da122:fb8e445e:55b853e0:81bd0a3e name=positron:pool"
        )
    )
    with patch.dict(mdadm.__salt__, {"cmd.run_stdout": mock}):
        assert mdadm.examine("/dev/md0") == {
            "ARRAY /dev/md/pool metadata": (
                "1.2 UUID=567da122:fb8e445e:55b853e0:81bd0a3e" " name=positron:pool"
            )
        }
        mock.assert_called_with(
            "mdadm -Y -E /dev/md0", ignore_retcode=False, python_shell=False
        )


def test_examine_quiet():
    """
    Test for mdadm_raid.examine
    """
    mock = MagicMock(return_value="")
    with patch.dict(mdadm.__salt__, {"cmd.run_stdout": mock}):
        assert mdadm.examine("/dev/md0", quiet=True) == {}
        mock.assert_called_with(
            "mdadm -Y -E /dev/md0", ignore_retcode=True, python_shell=False
        )


def test_device_match_regex_pattern():
    assert re.match(
        mdadm._VOL_REGEX_PATTERN_MATCH.format("/dev/md/1"),
        "ARRAY /dev/md/1  metadata=1.2 UUID=51f245bc:a1402c8a:2d598e79:589c07cf"
        " name=tst-ob-001:1",
    )
    assert not re.match(
        mdadm._VOL_REGEX_PATTERN_MATCH.format("/dev/md/1"),
        "ARRAY /dev/md/10  metadata=1.2 UUID=51f245bc:a1402c8a:2d598e79:589c07cf"
        " name=tst-ob-001:1",
    )



"""
Integration tests for renderer functions
"""

import pytest

from tests.support.case import ModuleCase


@pytest.mark.windows_whitelisted
class TestJinjaRenderer(ModuleCase):
    """
    Validate that ordering works correctly
    """

    @pytest.mark.slow_test
    def test_dot_notation(self):
        """
        Test the Jinja dot-notation syntax for calling execution modules
        """
        ret = self.run_function("state.sls", ["jinja_dot_notation"])
        for state_ret in ret.values():
            self.assertTrue(state_ret["result"])

    @pytest.mark.flaky(max_runs=4)
    @pytest.mark.slow_test
    def test_salt_contains_function(self):
        """
        Test if we are able to check if a function exists inside the "salt"
        wrapper (AliasLoader) which is available on Jinja templates.
        """
        ret = self.run_function("state.sls", ["jinja_salt_contains_function"])
        for state_ret in ret.values():
            self.assertTrue(state_ret["result"])



"""
    :codeauthor: Thomas Stoner <tmstoner@cisco.com>
"""

# Copyright (c) 2018 Cisco and/or its affiliates.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from tests.unit.modules.nxos.nxos_platform import NXOSPlatform

# pylint: disable-msg=C0103


class N95KPlatform(NXOSPlatform):
    """Cisco Systems N9K Platform Unit Test Object"""

    chassis = "Nexus9000 C9508 (8 Slot) Chassis"

    # Captured output from: show install all impact nxos <image>

    show_install_all_impact = """
Installer will perform impact only check. Please wait.

Verifying image bootflash:/$IMAGE for boot variable "nxos".
[####################] 100% -- SUCCESS

Verifying image type.
[####################] 100% -- SUCCESS

Preparing "bios" version info using image bootflash:/$IMAGE.
[####################] 100% -- SUCCESS

Preparing "nxos" version info using image bootflash:/$IMAGE.
[####################] 100% -- SUCCESS

Performing module support checks.
[####################] 100% -- SUCCESS

Notifying services about system upgrade.
[####################] 100% -- SUCCESS



Compatibility check is done:
Module  bootable          Impact  Install-type  Reason
------  --------  --------------  ------------  ------
     1       yes  non-disruptive          none
    22       yes  non-disruptive          none
    24       yes  non-disruptive          none
    26       yes  non-disruptive          none
    28       yes  non-disruptive          none
    29       yes  non-disruptive          none
    30       yes  non-disruptive          none



Images will be upgraded according to following table:
Module       Image                  Running-Version(pri:alt)           New-Version  Upg-Required
------  ----------  ----------------------------------------  --------------------  ------------
     1       lcn9k                               $CVER           $NVER            $REQ
     1        bios                       v01.48(00:v01.42(00             v01.48(00            no
    22       lcn9k                               $CVER           $NVER            $REQ
    22        bios                       v01.48(00:v01.42(00             v01.48(00            no
    24       lcn9k                               $CVER           $NVER            $REQ
    24        bios                       v01.48(00:v01.42(00             v01.48(00            no
    26       lcn9k                               $CVER           $NVER            $REQ
    26        bios                       v01.48(00:v01.42(00             v01.48(00            no
    28        nxos                               $CVER           $NVER            $REQ
    28        bios     v08.32(10/18/2016):v08.06(09/10/2014)    v08.32(10/18/2016)            no
    29       lcn9k                               $CVER           $NVER            $REQ
    29        bios                       v01.48(00:v01.42(00             v01.48(00            no
    30       lcn9k                               $CVER           $NVER            $REQ
    30        bios                       v01.48(00:v01.42(00             v01.48(00            no
"""

    # Captured output from: show install all impact nxos <image> non-disruptive

    show_install_all_impact_non_disruptive = """
Installer will perform impact only check. Please wait.

Verifying image bootflash:/$IMAGE for boot variable "nxos".
[####################] 100% -- SUCCESS

Verifying image type.
[####################] 100% -- SUCCESS

Preparing "bios" version info using image bootflash:/$IMAGE.
[####################] 100% -- SUCCESS

Preparing "nxos" version info using image bootflash:/$IMAGE.
[####################] 100% -- SUCCESS

Performing module support checks.
[####################] 100% -- SUCCESS

Notifying services about system upgrade.
[####################] 100% -- SUCCESS



Compatibility check is done:
Module  bootable          Impact  Install-type  Reason
------  --------  --------------  ------------  ------
     1       yes  non-disruptive          none
    22       yes  non-disruptive          none
    24       yes  non-disruptive          none
    26       yes  non-disruptive          none
    28       yes  non-disruptive          none
    29       yes  non-disruptive          none
    30       yes  non-disruptive          none



Images will be upgraded according to following table:
Module       Image                  Running-Version(pri:alt)           New-Version  Upg-Required
------  ----------  ----------------------------------------  --------------------  ------------
     1       lcn9k                                    $CVER                $NVER            $REQ
     1        bios                       v01.48(00:v01.42(00             v01.48(00            no
    22       lcn9k                                    $CVER                $NVER            $REQ
    22        bios                       v01.48(00:v01.42(00             v01.48(00            no
    24       lcn9k                                    $CVER                $NVER            $REQ
    24        bios                       v01.48(00:v01.42(00             v01.48(00            no
    26       lcn9k                                    $CVER                $NVER            $REQ
    26        bios                       v01.48(00:v01.42(00             v01.48(00            no
    28        nxos                                    $CVER                $NVER            $REQ
    28        bios     v08.35(08/31/2018):v08.06(09/10/2014)    v08.35(08/31/2018)            no
    29       lcn9k                                    $CVER                $NVER            $REQ
    29        bios                       v01.48(00:v01.42(00             v01.48(00            no
    30       lcn9k                                    $CVER                $NVER            $REQ
    30        bios                       v01.48(00:v01.42(00             v01.48(00            no
"""

    # Captured output from: install all nxos <image> non-disruptive

    install_all_non_disruptive_success = """
Installer will perform compatibility check first. Please wait.

Verifying image bootflash:/$IMAGE for boot variable "nxos".
[####################] 100% -- SUCCESS

Verifying image type.
[####################] 100% -- SUCCESS

Preparing "bios" version info using image bootflash:/$IMAGE.
[####################] 100% -- SUCCESS

Preparing "nxos" version info using image bootflash:/$IMAGE.
[####################] 100% -- SUCCESS

Performing module support checks.
[####################] 100% -- SUCCESS

Notifying services about system upgrade.
[####################] 100% -- SUCCESS



Compatibility check is done:
Module  bootable          Impact  Install-type  Reason
------  --------  --------------  ------------  ------
     1       yes  non-disruptive          none
    22       yes  non-disruptive          none
    24       yes  non-disruptive          none
    26       yes  non-disruptive          none
    28       yes  non-disruptive          none
    29       yes  non-disruptive          none
    30       yes  non-disruptive          none



Images will be upgraded according to following table:
Module       Image                  Running-Version(pri:alt)           New-Version  Upg-Required
------  ----------  ----------------------------------------  --------------------  ------------
     1       lcn9k                               $CVER           $NVER            $REQ
     1        bios                       v01.48(00:v01.42(00             v01.48(00            no
    22       lcn9k                               $CVER           $NVER            $REQ
    22        bios                       v01.48(00:v01.42(00             v01.48(00            no
    24       lcn9k                               $CVER           $NVER            $REQ
    24        bios                       v01.48(00:v01.42(00             v01.48(00            no
    26       lcn9k                               $CVER           $NVER            $REQ
    26        bios                       v01.48(00:v01.42(00             v01.48(00            no
    28        nxos                               $CVER           $NVER            $REQ
    28        bios     v08.32(10/18/2016):v08.06(09/10/2014)    v08.32(10/18/2016)            no
    29       lcn9k                               $CVER           $NVER            $REQ
    29        bios                       v01.48(00:v01.42(00             v01.48(00            no
    30       lcn9k                               $CVER           $NVER            $REQ
    30        bios                       v01.48(00:v01.42(00             v01.48(00            no


Install is in progress, please wait.

Performing runtime checks.
[####################] 100% -- SUCCESS

Setting boot variables.
[####################] 100% -- SUCCESS

Performing configuration copy.
[####################] 100% -- SUCCESS

Module 1: Refreshing compact flash and upgrading bios/loader/bootrom.
Warning: please do not remove or power off the module at this time.
[####################] 100% -- SUCCESS

Module 22: Refreshing compact flash and upgrading bios/loader/bootrom.
Warning: please do not remove or power off the module at this time.
[####################] 100% -- SUCCESS

Module 24: Refreshing compact flash and upgrading bios/loader/bootrom.
Warning: please do not remove or power off the module at this time.
[####################] 100% -- SUCCESS

Module 26: Refreshing compact flash and upgrading bios/loader/bootrom.
Warning: please do not remove or power off the module at this time.
[####################] 100% -- SUCCESS

Module 28: Refreshing compact flash and upgrading bios/loader/bootrom.
Warning: please do not remove or power off the module at this time.
[####################] 100% -- SUCCESS

Module 29: Refreshing compact flash and upgrading bios/loader/bootrom.
Warning: please do not remove or power off the module at this time.
[####################] 100% -- SUCCESS

Module 30: Refreshing compact flash and upgrading bios/loader/bootrom.
Warning: please do not remove or power off the module at this time.
[####################] 100% -- SUCCESS


Install has been successful.
"""

    # Captured output from: install all nxos <image>

    install_all_disruptive_success = """
Installer will perform compatibility check first. Please wait.
Installer is forced disruptive

Verifying image bootflash:/$IMAGE for boot variable "nxos".
[####################] 100% -- SUCCESS

Verifying image type.
[####################] 100% -- SUCCESS

Preparing "bios" version info using image bootflash:/$IMAGE.
[####################] 100% -- SUCCESS

Preparing "nxos" version info using image bootflash:/$IMAGE.
[####################] 100% -- SUCCESS

Performing module support checks.
[####################] 100% -- SUCCESS

Notifying services about system upgrade.
[####################] 100% -- SUCCESS



Compatibility check is done:
Module  bootable          Impact  Install-type  Reason
------  --------  --------------  ------------  ------
     1       yes      disruptive         reset  default upgrade is not hitless
    22       yes      disruptive         reset  default upgrade is not hitless
    24       yes      disruptive         reset  default upgrade is not hitless
    26       yes      disruptive         reset  default upgrade is not hitless
    28       yes      disruptive         reset  default upgrade is not hitless
    29       yes      disruptive         reset  default upgrade is not hitless
    30       yes      disruptive         reset  default upgrade is not hitless



Images will be upgraded according to following table:
Module       Image                  Running-Version(pri:alt)           New-Version  Upg-Required
------  ----------  ----------------------------------------  --------------------  ------------
     1       lcn9k                               $CVER                $NVER           $REQ
     1        bios                       v01.48(00:v01.42(00             v01.48(00            no
    22       lcn9k                               $CVER                $NVER           $REQ
    22        bios                       v01.48(00:v01.42(00             v01.48(00            no
    24       lcn9k                               $CVER                $NVER           $REQ
    24        bios                       v01.48(00:v01.42(00             v01.48(00            no
    26       lcn9k                               $CVER                $NVER           $REQ
    26        bios                       v01.48(00:v01.42(00             v01.48(00            no
    28        nxos                               $CVER                $NVER           $REQ
    28        bios     v08.32(10/18/2016):v08.06(09/10/2014)    v08.35(08/31/2018)           yes
    29       lcn9k                               $CVER                $NVER           $REQ
    29        bios                       v01.48(00:v01.42(00             v01.48(00            no
    30       lcn9k                               $CVER                $NVER           $REQ
    30        bios                       v01.48(00:v01.42(00             v01.48(00            no


Switch will be reloaded for disruptive upgrade.

Install is in progress, please wait.

Performing runtime checks.
[####################] 100% -- SUCCESS

Setting boot variables.
[####################] 100% -- SUCCESS

Performing configuration copy.
[####################] 100% -- SUCCESS

Module 1: Refreshing compact flash and upgrading bios/loader/bootrom.
Warning: please do not remove or power off the module at this time.
[####################] 100% -- SUCCESS

Module 22: Refreshing compact flash and upgrading bios/loader/bootrom.
Warning: please do not remove or power off the module at this time.
[####################] 100% -- SUCCESS

Module 24: Refreshing compact flash and upgrading bios/loader/bootrom.
Warning: please do not remove or power off the module at this time.
[####################] 100% -- SUCCESS

Module 26: Refreshing compact flash and upgrading bios/loader/bootrom.
Warning: please do not remove or power off the module at this time.
[####################] 100% -- SUCCESS

Module 28: Refreshing compact flash and upgrading bios/loader/bootrom.
Warning: please do not remove or power off the module at this time.
[####################] 100% -- SUCCESS

Module 29: Refreshing compact flash and upgrading bios/loader/bootrom.
Warning: please do not remove or power off the module at this time.
[####################] 100% -- SUCCESS

Module 30: Refreshing compact flash and upgrading bios/loader/bootrom.
Warning: please do not remove or power off the module at this time.
[####################] 100% -- SUCCESS


Finishing the upgrade, switch will reboot in 10 seconds.
"""



"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>

    Test cases for salt.modules.npm
"""

import textwrap

import pytest

import salt.modules.npm as npm
import salt.utils.json
from salt.exceptions import CommandExecutionError
from tests.support.mock import MagicMock, patch


@pytest.fixture
def configure_loader_modules():
    with patch("salt.modules.npm._check_valid_version", MagicMock(return_value=True)):
        return {npm: {}}


# 'install' function tests: 4


def test_install():
    """
    Test if it installs an NPM package.
    """
    mock = MagicMock(return_value={"retcode": 1, "stderr": "error"})
    with patch.dict(npm.__salt__, {"cmd.run_all": mock}):
        pytest.raises(CommandExecutionError, npm.install, "coffee-script")

    # This is at least somewhat closer to the actual output format.
    mock_json_out = textwrap.dedent(
        """\
    [
      {
        "salt": "SALT"
      }
    ]"""
    )

    # Successful run, expected output format
    mock = MagicMock(return_value={"retcode": 0, "stderr": "", "stdout": mock_json_out})
    with patch.dict(npm.__salt__, {"cmd.run_all": mock}):
        assert npm.install("coffee-script") == [{"salt": "SALT"}]

    mock_json_out_extra = textwrap.dedent(
        """\
    Compilation output here

    [bcrypt] Success: "/tmp/node_modules/bcrypt/foo" is installed via remote"
    [grpc] Success: "/usr/lib/node_modules/@foo/bar" is installed via remote"
    [
       {
          "from" : "express@",
          "name" : "express",
          "dependencies" : {
             "escape-html" : {
                "from" : "escape-html@~1.0.3",
                "dependencies" : {},
                "version" : "1.0.3"
             }
          },
          "version" : "4.16.3"
       }
    ]"""
    )
    extra_expected = [
        {
            "dependencies": {
                "escape-html": {
                    "dependencies": {},
                    "from": "escape-html@~1.0.3",
                    "version": "1.0.3",
                }
            },
            "from": "express@",
            "name": "express",
            "version": "4.16.3",
        }
    ]

    # Successful run, expected output format with additional leading text
    mock = MagicMock(
        return_value={"retcode": 0, "stderr": "", "stdout": mock_json_out_extra}
    )
    with patch.dict(npm.__salt__, {"cmd.run_all": mock}):
        assert npm.install("coffee-script") == extra_expected

    # Successful run, unexpected output format
    mock = MagicMock(return_value={"retcode": 0, "stderr": "", "stdout": "SALT"})
    with patch.dict(npm.__salt__, {"cmd.run_all": mock}):
        mock_err = MagicMock(side_effect=ValueError())
        # When JSON isn't successfully parsed, return should equal input
        with patch.object(salt.utils.json, "loads", mock_err):
            assert npm.install("coffee-script") == "SALT"


# 'uninstall' function tests: 1


def test_uninstall():
    """
    Test if it uninstalls an NPM package.
    """
    mock = MagicMock(return_value={"retcode": 1, "stderr": "error"})
    with patch.dict(npm.__salt__, {"cmd.run_all": mock}):
        assert not npm.uninstall("coffee-script")

    mock = MagicMock(return_value={"retcode": 0, "stderr": ""})
    with patch.dict(npm.__salt__, {"cmd.run_all": mock}):
        assert npm.uninstall("coffee-script")


# 'list_' function tests: 1


def test_list():
    """
    Test if it list installed NPM packages.
    """
    mock = MagicMock(return_value={"retcode": 1, "stderr": "error"})
    with patch.dict(npm.__salt__, {"cmd.run_all": mock}):
        pytest.raises(CommandExecutionError, npm.list_, "coffee-script")

    mock = MagicMock(
        return_value={
            "retcode": 0,
            "stderr": "error",
            "stdout": '{"salt": ["SALT"]}',
        }
    )
    with patch.dict(npm.__salt__, {"cmd.run_all": mock}):
        mock_err = MagicMock(return_value={"dependencies": "SALT"})
        with patch.object(salt.utils.json, "loads", mock_err):
            assert npm.list_("coffee-script") == "SALT"


# 'cache_clean' function tests: 1


def test_cache_clean():
    """
    Test if it cleans the cached NPM packages.
    """
    mock = MagicMock(return_value={"retcode": 1, "stderr": "error"})
    with patch.dict(npm.__salt__, {"cmd.run_all": mock}):
        assert not npm.cache_clean()

    mock = MagicMock(return_value={"retcode": 0})
    with patch.dict(npm.__salt__, {"cmd.run_all": mock}):
        assert npm.cache_clean()

    mock = MagicMock(return_value={"retcode": 0})
    with patch.dict(npm.__salt__, {"cmd.run_all": mock}):
        assert npm.cache_clean("coffee-script")


# 'cache_list' function tests: 1


def test_cache_list():
    """
    Test if it lists the NPM cache.
    """
    mock = MagicMock(return_value={"retcode": 1, "stderr": "error"})
    with patch.dict(npm.__salt__, {"cmd.run_all": mock}):
        pytest.raises(CommandExecutionError, npm.cache_list)

    mock = MagicMock(
        return_value={"retcode": 0, "stderr": "error", "stdout": ["~/.npm"]}
    )
    with patch.dict(npm.__salt__, {"cmd.run_all": mock}):
        assert npm.cache_list() == ["~/.npm"]

    mock = MagicMock(return_value={"retcode": 0, "stderr": "error", "stdout": ""})
    with patch.dict(npm.__salt__, {"cmd.run_all": mock}):
        assert npm.cache_list("coffee-script") == ""


# 'cache_path' function tests: 1


def test_cache_path():
    """
    Test if it prints the NPM cache path.
    """
    mock = MagicMock(return_value={"retcode": 1, "stderr": "error"})
    with patch.dict(npm.__salt__, {"cmd.run_all": mock}):
        assert npm.cache_path() == "error"

    mock = MagicMock(
        return_value={"retcode": 0, "stderr": "error", "stdout": "/User/salt/.npm"}
    )
    with patch.dict(npm.__salt__, {"cmd.run_all": mock}):
        assert npm.cache_path() == "/User/salt/.npm"



"""
integration tests for mac_shadow
"""

import datetime
import types

import pytest
from saltfactories.utils import random_string

from salt.exceptions import CommandExecutionError

pytestmark = [
    pytest.mark.skip_if_binaries_missing("dscl", "pwpolicy"),
    pytest.mark.slow_test,
    pytest.mark.skip_if_not_root,
    pytest.mark.skip_unless_on_darwin,
]


@pytest.fixture(scope="module")
def shadow(modules):
    return modules.shadow


@pytest.fixture
def accounts():
    with pytest.helpers.create_account(create_group=True) as _account:
        yield types.SimpleNamespace(
            existing=_account.username,
            non_existing=random_string("account-", lowercase=False),
        )


def test_info(shadow, accounts):
    """
    Test shadow.info
    """
    # Correct Functionality
    ret = shadow.info(accounts.existing)
    assert ret["name"] == accounts.existing

    # User does not exist
    ret = shadow.info(accounts.non_existing)
    assert ret["name"] == ""


def test_get_last_change(shadow, accounts):
    """
    Test shadow.get_last_change
    """
    # Correct Functionality
    text_date = shadow.get_last_change(accounts.existing)
    assert text_date != "Invalid Timestamp"
    obj_date = datetime.datetime.strptime(text_date, "%Y-%m-%d %H:%M:%S")
    assert isinstance(obj_date, datetime.date)

    # User does not exist
    with pytest.raises(CommandExecutionError) as exc:
        shadow.get_last_change(accounts.non_existing)
        assert f"ERROR: User not found: {accounts.non_existing}" in str(exc.value)


def test_get_login_failed_last(shadow, accounts):
    """
    Test shadow.get_login_failed_last
    """
    # Correct Functionality
    text_date = shadow.get_login_failed_last(accounts.existing)
    assert text_date != "Invalid Timestamp"
    obj_date = datetime.datetime.strptime(text_date, "%Y-%m-%d %H:%M:%S")
    assert isinstance(obj_date, datetime.date)

    # User does not exist
    with pytest.raises(CommandExecutionError) as exc:
        shadow.get_login_failed_last(accounts)
        assert f"ERROR: User not found: {accounts.non_existing}" in str(exc.value)


def test_get_login_failed_count(shadow, accounts):
    """
    Test shadow.get_login_failed_count
    """
    # Correct Functionality
    assert shadow.get_login_failed_count(accounts.existing) == "0"

    # User does not exist
    with pytest.raises(CommandExecutionError) as exc:
        shadow.get_login_failed_count(accounts.non_existing)
        assert f"ERROR: User not found: {accounts.non_existing}" in str(exc.value)


def test_get_set_maxdays(shadow, accounts):
    """
    Test shadow.get_maxdays
    Test shadow.set_maxdays
    """
    # Correct Functionality
    assert shadow.set_maxdays(accounts.existing, 20)
    assert shadow.get_maxdays(accounts.existing) == 20

    # User does not exist
    with pytest.raises(CommandExecutionError) as exc:
        shadow.set_maxdays(accounts.non_existing, 7)
        assert f"ERROR: User not found: {accounts.non_existing}" in str(exc.value)

    with pytest.raises(CommandExecutionError) as exc:
        shadow.get_maxdays(accounts.non_existing)
        assert f"ERROR: User not found: {accounts.non_existing}" in str(exc.value)


def test_get_set_change(shadow, accounts):
    """
    Test shadow.get_change
    Test shadow.set_change
    """
    # Correct Functionality
    assert shadow.set_change(accounts.existing, "02/11/2011")
    assert shadow.get_change(accounts.existing) == "02/11/2011"

    # User does not exist
    with pytest.raises(CommandExecutionError) as exc:
        shadow.set_change(accounts.non_existing, "02/11/2012")
        assert f"ERROR: User not found: {accounts.non_existing}" in str(exc.value)

    with pytest.raises(CommandExecutionError) as exc:
        shadow.get_change(accounts.non_existing)
        assert f"ERROR: User not found: {accounts.non_existing}" in str(exc.value)


def test_get_set_expire(shadow, accounts):
    """
    Test shadow.get_expire
    Test shadow.set_expire
    """
    # Correct Functionality
    assert shadow.set_expire(accounts.existing, "02/11/2011")
    assert shadow.get_expire(accounts.existing) == "02/11/2011"

    # User does not exist
    with pytest.raises(CommandExecutionError) as exc:
        shadow.set_expire(accounts.non_existing, "02/11/2012")
        assert f"ERROR: User not found: {accounts.non_existing}" in str(exc.value)

    with pytest.raises(CommandExecutionError) as exc:
        shadow.get_expire(accounts.non_existing)
        assert f"ERROR: User not found: {accounts.non_existing}" in str(exc.value)


def test_del_password(shadow, accounts):
    """
    Test shadow.del_password
    """
    # Correct Functionality
    assert shadow.del_password(accounts.existing)
    assert shadow.info(accounts.existing)["passwd"] == "*"

    # User does not exist
    with pytest.raises(CommandExecutionError) as exc:
        shadow.del_password(accounts.non_existing)
        assert f"ERROR: User not found: {accounts.non_existing}" in str(exc.value)


def test_set_password(shadow, accounts):
    """
    Test shadow.set_password
    """
    # Correct Functionality
    assert shadow.set_password(accounts.existing, "Pa$$W0rd")

    # User does not exist
    with pytest.raises(CommandExecutionError) as exc:
        shadow.set_password(accounts.non_existing, "P@SSw0rd")
        assert f"ERROR: User not found: {accounts.non_existing}" in str(exc.value)



import logging
import pathlib
import subprocess

import pytest

from tests.conftest import FIPS_TESTRUN

log = logging.getLogger(__name__)


@pytest.fixture(scope="package")
def master_alive_interval():
    return 5


class HostsFile:
    """
    Simple helper class for tests that need to modify /etc/hosts.
    """

    def __init__(self, path, orig_text):
        self._path = path
        self._orig_text = orig_text

    @property
    def orig_text(self):
        return self._orig_text

    def __getattr__(self, key):
        if key in ["_path", "_orig_text", "orig_text"]:
            return self.__getattribute__(key)
        return getattr(self._path, key)


@pytest.fixture
def etc_hosts():
    hosts = pathlib.Path("/etc/hosts")
    orig_text = hosts.read_text(encoding="utf-8")
    hosts = HostsFile(hosts, orig_text)
    try:
        yield hosts
    finally:
        hosts.write_text(orig_text)


@pytest.fixture(scope="package")
def master(request, salt_factories):

    subprocess.check_output(["ip", "addr", "add", "172.16.0.1/32", "dev", "lo"])

    config_defaults = {
        "open_mode": True,
        "transport": request.config.getoption("--transport"),
    }
    config_overrides = {
        "worker_pools_enabled": True,
        "worker_pools": {
            "fast": {
                "worker_count": 2,
                "commands": [
                    "test.ping",
                    "test.echo",
                    "test.fib",
                    "grains.items",
                    "sys.doc",
                    "pillar.items",
                    "runner.test.arg",
                    "auth",
                ],
            },
            "general": {
                "worker_count": 3,
                "commands": ["*"],
            },
        },
        "interface": "0.0.0.0",
        "fips_mode": FIPS_TESTRUN,
        "publish_signing_algorithm": (
            "PKCS1v15-SHA224" if FIPS_TESTRUN else "PKCS1v15-SHA1"
        ),
    }
    factory = salt_factories.salt_master_daemon(
        "master",
        defaults=config_defaults,
        overrides=config_overrides,
        extra_cli_arguments_after_first_start_failure=["--log-level=info"],
    )
    with factory.started(start_timeout=180):
        yield factory

    try:
        subprocess.check_output(["ip", "addr", "del", "172.16.0.1/32", "dev", "lo"])
    except subprocess.CalledProcessError:
        pass


@pytest.fixture(scope="package")
def salt_cli(master):
    return master.salt_cli(timeout=180)


@pytest.fixture(scope="package")
def minion(master, master_alive_interval):
    config_defaults = {
        "transport": master.config["transport"],
    }
    port = master.config["ret_port"]
    config_overrides = {
        "worker_pools_enabled": True,
        "worker_pools": {
            "fast": {
                "worker_count": 2,
                "commands": [
                    "test.ping",
                    "test.echo",
                    "test.fib",
                    "grains.items",
                    "sys.doc",
                    "pillar.items",
                    "runner.test.arg",
                    "auth",
                ],
            },
            "general": {
                "worker_count": 3,
                "commands": ["*"],
            },
        },
        "master": f"master.local:{port}",
        "publish_port": master.config["publish_port"],
        "master_alive_interval": master_alive_interval,
        "fips_mode": FIPS_TESTRUN,
        "encryption_algorithm": "OAEP-SHA224" if FIPS_TESTRUN else "OAEP-SHA1",
        "signing_algorithm": "PKCS1v15-SHA224" if FIPS_TESTRUN else "PKCS1v15-SHA1",
    }
    factory = master.salt_minion_daemon(
        "minion",
        defaults=config_defaults,
        overrides=config_overrides,
        extra_cli_arguments_after_first_start_failure=["--log-level=info"],
    )
    return factory



"""
    :codeauthor: Tyler Johnson <tjohnson@saltstack.com>
"""

import pytest

import salt.states.pkgrepo as pkgrepo
from tests.support.mock import MagicMock, patch


@pytest.fixture
def configure_loader_modules():
    return {
        pkgrepo: {
            "__opts__": {"test": True},
            "__grains__": {"os": "", "os_family": ""},
        }
    }


def test_name_change():
    """
    Test when only the key_url is changed that a change is triggered
    """
    kwargs = {
        "name": "deb http://apt.example.com/{{grains['os'] | lower}} {{grains['oscodename']}} main",
        "disabled": False,
        "key_url": "https://mock/changed_gpg.key",
    }

    new = kwargs.copy()
    new["name"] = (
        "deb [arch=amd64] http://apt.example.com/{{grains['os'] | lower}} {{grains['oscodename']}} main"
    )

    with patch.dict(pkgrepo.__salt__, {"pkg.get_repo": MagicMock(return_value=kwargs)}):
        ret = pkgrepo.managed(**new)
        assert ret["changes"] == {"name": {"old": kwargs["name"], "new": new["name"]}}


def test_new_key_url():
    """
    Test when only the key_url is changed that a change is triggered
    """
    kwargs = {
        "name": "deb https://mock/ sid main",
        "disabled": False,
    }
    key_url = "https://mock/changed_gpg.key"

    with patch.dict(pkgrepo.__salt__, {"pkg.get_repo": MagicMock(return_value=kwargs)}):
        ret = pkgrepo.managed(key_url=key_url, **kwargs)
        assert ret["changes"] == {"key_url": {"old": None, "new": key_url}}


def test_update_key_url():
    """
    Test when only the key_url is changed that a change is triggered
    """
    kwargs = {
        "name": "deb https://mock/ sid main",
        "gpgcheck": 1,
        "disabled": False,
        "key_url": "https://mock/gpg.key",
    }
    changed_kwargs = kwargs.copy()
    changed_kwargs["key_url"] = "https://mock/gpg2.key"

    with patch.dict(pkgrepo.__salt__, {"pkg.get_repo": MagicMock(return_value=kwargs)}):
        ret = pkgrepo.managed(**changed_kwargs)
        assert "key_url" in ret["changes"], "Expected a change to key_url"
        assert ret["changes"] == {
            "key_url": {"old": kwargs["key_url"], "new": changed_kwargs["key_url"]}
        }


def test_managed_insecure_key():
    """
    Test when only the key_url is changed that a change is triggered
    """
    kwargs = {
        "name": "deb http://mock/ sid main",
        "gpgcheck": 1,
        "disabled": False,
        "key_url": "http://mock/gpg.key",
        "allow_insecure_key": False,
    }
    with patch.dict(pkgrepo.__salt__, {"pkg.get_repo": MagicMock(return_value=kwargs)}):
        ret = pkgrepo.managed(**kwargs)
        assert ret["result"] is False
        assert (
            ret["comment"]
            == "Cannot have 'key_url' using http with 'allow_insecure_key' set to True"
        )
